@@ -24,7 +24,6 @@ import java.nio.ByteBuffer;
 
 import io.netty.buffer.ByteBuf;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffer;
-import org.apache.activemq.artemis.api.core.ActiveMQBuffers;
 import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.core.client.ActiveMQClientLogger;
@@ -36,7 +35,7 @@ import org.apache.activemq.artemis.utils.UTF8Util;
 
 final class CompressedLargeMessageControllerImpl implements LargeMessageController {
 
-   private static final String OPERATION_NOT_SUPPORTED = "Operation not supported";
+   private static final String OPERATION_NOT_SUPPORTED = "Operation not supported over compressed large messages";
 
    private final LargeMessageController bufferDelegate;
 
@@ -44,9 +43,6 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       this.bufferDelegate = bufferDelegate;
    }
 
-   /**
-    *
-    */
    @Override
    public void discardUnusedPackets() {
       bufferDelegate.discardUnusedPackets();
@@ -89,6 +85,11 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       return bufferDelegate.waitCompletion(timeWait);
    }
 
+   @Override
+   public LargeMessageControllerImpl.LargeData take() throws InterruptedException {
+      return bufferDelegate.take();
+   }
+
    @Override
    public int capacity() {
       return -1;
@@ -110,10 +111,6 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       return dataInput;
    }
 
-   private void positioningNotSupported() {
-      throw new IllegalStateException("Position not supported over compressed large messages");
-   }
-
    @Override
    public byte readByte() {
       try {
@@ -125,41 +122,37 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
 
    @Override
    public byte getByte(final int index) {
-      positioningNotSupported();
-      return 0;
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
    public void getBytes(final int index, final ActiveMQBuffer dst, final int dstIndex, final int length) {
-      positioningNotSupported();
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
    public void getBytes(final int index, final byte[] dst, final int dstIndex, final int length) {
-      positioningNotSupported();
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
    public void getBytes(final int index, final ByteBuffer dst) {
-      positioningNotSupported();
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
    public int getInt(final int index) {
-      positioningNotSupported();
-      return 0;
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
    public long getLong(final int index) {
-      positioningNotSupported();
-      return 0;
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
    public short getShort(final int index) {
-      positioningNotSupported();
-      return 0;
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
@@ -235,7 +228,7 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
 
    @Override
    public void setIndex(final int readerIndex, final int writerIndex) {
-      positioningNotSupported();
+      throw new IllegalStateException(OPERATION_NOT_SUPPORTED);
    }
 
    @Override
@@ -375,6 +368,16 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       }
    }
 
+   @Override
+   public Integer readNullableInt() {
+      int b = readByte();
+      if (b == DataConstants.NULL) {
+         return null;
+      } else {
+         return readInt();
+      }
+   }
+
    @Override
    public long readUnsignedInt() {
       return readInt() & 0xFFFFFFFFL;
@@ -389,6 +392,16 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       }
    }
 
+   @Override
+   public Long readNullableLong() {
+      int b = readByte();
+      if (b == DataConstants.NULL) {
+         return null;
+      } else {
+         return readLong();
+      }
+   }
+
    @Override
    public void readBytes(final byte[] dst, final int dstIndex, final int length) {
       try {
@@ -487,11 +500,21 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
    }
 
+   @Override
+   public void writeNullableInt(final Integer value) {
+      throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
+   }
+
    @Override
    public void writeLong(final long value) {
       throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
    }
 
+   @Override
+   public void writeNullableLong(final Long value) {
+      throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
+   }
+
    @Override
    public void writeBytes(final byte[] src, final int srcIndex, final int length) {
       throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
@@ -532,6 +555,16 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       return readByte() != 0;
    }
 
+   @Override
+   public Boolean readNullableBoolean() {
+      int b = readByte();
+      if (b == DataConstants.NULL) {
+         return null;
+      } else {
+         return readBoolean();
+      }
+   }
+
    @Override
    public char readChar() {
       return (char) readShort();
@@ -552,13 +585,6 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       return Float.intBitsToFloat(getInt(index));
    }
 
-   @Override
-   public ActiveMQBuffer readBytes(final int length) {
-      byte[] bytesToGet = new byte[length];
-      readBytes(bytesToGet);
-      return ActiveMQBuffers.wrappedBuffer(bytesToGet);
-   }
-
    @Override
    public double readDouble() {
       return Double.longBitsToDouble(readLong());
@@ -594,7 +620,7 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       int len = readInt();
       byte[] data = new byte[len];
       readBytes(data);
-      return new SimpleString(data);
+      return SimpleString.of(data);
    }
 
    @Override
@@ -624,6 +650,11 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
    }
 
+   @Override
+   public void writeNullableBoolean(final Boolean val) {
+      throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
+   }
+
    @Override
    public void writeChar(final char val) {
       throw new IllegalAccessError(OPERATION_NOT_SUPPORTED);
@@ -676,8 +707,6 @@ final class CompressedLargeMessageControllerImpl implements LargeMessageControll
       throw new UnsupportedOperationException();
    }
 
-   // Inner classes -------------------------------------------------
-
    @Override
    public ByteBuf byteBuf() {
       return null;
