@@ -1,12 +1,12 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. See the NOTICE file distributed with
+ * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
+ * the License.  You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.activemq.artemis.core.message.impl;
 
 import javax.management.openmbean.ArrayType;
@@ -64,8 +63,6 @@ import java.lang.invoke.MethodHandles;
 
 import static org.apache.activemq.artemis.utils.ByteUtil.ensureExactWritable;
 
-/** Note: you shouldn't change properties using multi-threads. Change your properties before you can send it to multiple
- *  consumers */
 public class CoreMessage extends RefCountMessage implements ICoreMessage {
 
    public static final int BUFFER_HEADER_SPACE = PacketImpl.PACKET_HEADERS_SIZE;
@@ -77,8 +74,10 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    // There's an integer with the number of bytes for the body
    public static final int BODY_OFFSET = DataConstants.SIZE_INT;
 
-   /** That is the readInto for the whole message, including properties..
-       it does not include the buffer for the Packet send and receive header on core protocol */
+   /**
+    * That is the readInto for the whole message, including properties. It does not include the buffer for the Packet
+    * send and receive header on core protocol
+    */
    protected ByteBuf buffer;
 
    private volatile boolean validBuffer = false;
@@ -97,8 +96,10 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
 
    protected boolean durable;
 
+   protected boolean paged;
+
    /**
-    * GMT milliseconds at which this message expires. 0 means never expires *
+    * GMT milliseconds at which this message expires. 0 means never expires
     */
    private long expiration;
 
@@ -124,18 +125,32 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
       this.coreMessageObjectPools = null;
    }
 
+   @Override
+   public void setPaged() {
+      this.paged = true;
+   }
+
+   @Override
+   public boolean isPaged() {
+      return paged;
+   }
+
    @Override
    public String getProtocolName() {
       return ActiveMQClient.DEFAULT_CORE_PROTOCOL;
    }
 
-   /** On core there's no delivery annotation */
+   /**
+    * On core there's no delivery annotation
+    */
    @Override
    public Object getAnnotation(SimpleString key) {
       return getObjectProperty(key);
    }
 
-   /** On core there's no delivery annotation */
+   /**
+    * On core there's no delivery annotation
+    */
    @Override
    public Object removeAnnotation(SimpleString key) {
       return removeProperty(key);
@@ -149,6 +164,14 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
       }
    }
 
+   @Override
+   public void clearAMQPProperties() {
+      final TypedProperties properties = this.properties;
+      if (properties != null && properties.clearAMQPProperties()) {
+         messageChanged();
+      }
+   }
+
    @Override
    public Persister<Message> getPersister() {
       return CoreMessagePersister.getInstance();
@@ -207,7 +230,9 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
       decode(false);
    }
 
-   /** This will fix the incoming body of 1.x messages */
+   /**
+    * This will fix the incoming body of 1.x messages
+    */
    @Override
    public void receiveBuffer_1X(ByteBuf buffer) {
       this.buffer = buffer;
@@ -229,9 +254,8 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    }
 
    /**
-    * This will return the proper buffer to represent the data of the Message. If compressed it will decompress.
-    * If large, it will read from the file or streaming.
-    * @return
+    * This will return the proper buffer to represent the data of the Message. If compressed it will decompress. If
+    * large, it will read from the file or streaming.
     */
    @Override
    public ActiveMQBuffer getDataBuffer() {
@@ -257,16 +281,17 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    }
 
    private ActiveMQBuffer getLargeMessageBuffer() throws ActiveMQException {
-      LargeBodyReader encoder = getLargeBodyReader();
-      encoder.open();
-      int bodySize = (int) encoder.getSize();
-      final ActiveMQBuffer buffer = new ChannelBufferWrapper(UnpooledByteBufAllocator.DEFAULT.heapBuffer(bodySize));
-      buffer.byteBuf().ensureWritable(bodySize);
-      final ByteBuffer nioBuffer = buffer.byteBuf().internalNioBuffer(0, bodySize);
-      encoder.readInto(nioBuffer);
-      buffer.writerIndex(bodySize);
-      encoder.close();
-      return buffer;
+      try (LargeBodyReader encoder = getLargeBodyReader()) {
+         encoder.open();
+         int bodySize = (int) encoder.getSize();
+         final ActiveMQBuffer buffer = new ChannelBufferWrapper(UnpooledByteBufAllocator.DEFAULT.heapBuffer(bodySize));
+         buffer.byteBuf().ensureWritable(bodySize);
+         final ByteBuffer nioBuffer = buffer.byteBuf().internalNioBuffer(0, bodySize);
+         encoder.readInto(nioBuffer);
+         buffer.writerIndex(bodySize);
+         encoder.close();
+         return buffer;
+      }
    }
 
    private ActiveMQBuffer inflate(ActiveMQBuffer buffer) throws DataFormatException {
@@ -302,7 +327,7 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
 
    @Override
    public CoreMessage setGroupID(String groupId) {
-      return this.setGroupID(SimpleString.toSimpleString(groupId, coreMessageObjectPools == null ? null : coreMessageObjectPools.getGroupIdStringSimpleStringPool()));
+      return this.setGroupID(SimpleString.of(groupId, coreMessageObjectPools == null ? null : coreMessageObjectPools.getGroupIdStringSimpleStringPool()));
    }
 
    @Override
@@ -327,7 +352,6 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    }
 
    /**
-    * @param sendBuffer
     * @param deliveryCount Some protocols (AMQP) will have this as part of the message. ignored on core
     */
    @Override
@@ -362,8 +386,8 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    public Long getScheduledDeliveryTime() {
       Object property = getProperties().getProperty(Message.HDR_SCHEDULED_DELIVERY_TIME);
 
-      if (property != null && property instanceof Number) {
-         return ((Number) property).longValue();
+      if (property != null && property instanceof Number number) {
+         return number.longValue();
       }
 
       return 0L;
@@ -466,8 +490,10 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
       }
    }
 
-   /** This method serves as a purpose of extension.
-    *   Large Message on a Core Message will have to set the messageID on the attached NewLargeMessage */
+   /**
+    * This method serves as a purpose of extension. Large Message on a Core Message will have to set the messageID on
+    * the attached NewLargeMessage
+    */
    protected void internalSetMessageID(final long messageID) {
       this.messageID = messageID;
    }
@@ -483,8 +509,8 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
       timestamp = msg.getTimestamp();
       priority = msg.getPriority();
 
-      if (msg instanceof CoreMessage) {
-         properties = new TypedProperties(((CoreMessage) msg).getProperties());
+      if (msg instanceof CoreMessage coreMessage) {
+         properties = new TypedProperties(coreMessage.getProperties());
       }
    }
 
@@ -610,7 +636,7 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
       try {
          TypedProperties properties = this.properties;
          if (properties == null) {
-            properties = new TypedProperties(INTERNAL_PROPERTY_NAMES_PREDICATE);
+            properties = new TypedProperties(INTERNAL_PROPERTY_NAMES_PREDICATE, AMQP_PROPERTY_PREDICATE);
             if (buffer != null && propertiesLocation >= 0) {
                final ByteBuf byteBuf = buffer.duplicate().readerIndex(propertiesLocation);
                properties.decode(byteBuf, coreMessageObjectPools == null ? null : coreMessageObjectPools.getPropertiesDecoderPools());
@@ -726,7 +752,7 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
          properties = null;
          propertiesLocation = buffer.readerIndex();
       } else {
-         properties = new TypedProperties(INTERNAL_PROPERTY_NAMES_PREDICATE);
+         properties = new TypedProperties(INTERNAL_PROPERTY_NAMES_PREDICATE, AMQP_PROPERTY_PREDICATE);
          properties.decode(buffer, pools == null ? null : pools.getPropertiesDecoderPools());
       }
    }
@@ -771,7 +797,8 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
          Long.BYTES +                                    // expiration
          Long.BYTES +                                    // timestamp
          Byte.BYTES;                                     // priority
-      synchronized (properties) {
+      properties.getReadLock().lock();
+      try {
          final int propertiesEncodeSize = properties.getEncodeSize();
          final int totalEncodedSize = headersSize + propertiesEncodeSize;
          ensureExactWritable(buffer, totalEncodedSize);
@@ -791,6 +818,8 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
          assert buffer.writerIndex() == initialWriterIndex + headersSize : "Bad Headers encode size estimation";
          final int realPropertiesEncodeSize = properties.encode(buffer);
          assert realPropertiesEncodeSize == propertiesEncodeSize : "TypedProperties has a wrong encode size estimation or is being modified concurrently";
+      } finally {
+         properties.getReadLock().unlock();
       }
    }
 
@@ -849,7 +878,7 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    @Override
    public CoreMessage setAddress(String address) {
       messageChanged();
-      this.address = SimpleString.toSimpleString(address, coreMessageObjectPools == null ? null : coreMessageObjectPools.getAddressStringSimpleStringPool());
+      this.address = SimpleString.of(address, coreMessageObjectPools == null ? null : coreMessageObjectPools.getAddressStringSimpleStringPool());
       return this;
    }
 
@@ -1151,9 +1180,9 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    }
 
    /**
-    * Differently from {@link #containsProperty(SimpleString)}, this method can save decoding the message,
-    * performing a search of the {@code key} property and falling back to {@link #containsProperty(SimpleString)}
-    * if not possible or if already decoded.
+    * Differently from {@link #containsProperty(SimpleString)}, this method can save decoding the message, performing a
+    * search of the {@code key} property and falling back to {@link #containsProperty(SimpleString)} if not possible or
+    * if already decoded.
     */
    public boolean searchProperty(SimpleString key) {
       Objects.requireNonNull(key, "key cannot be null");
@@ -1162,12 +1191,9 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
          return properties.containsProperty(key);
       }
       synchronized (this) {
-         final ByteBuf buffer = this.buffer;
+         final ByteBuf buffer = Objects.requireNonNull(this.buffer, "buffer cannot be null");
          // acquiring the lock here, although heavy-weight, is the safer way to do this,
          // because we cannot trust that a racing thread won't modify buffer
-         if (buffer == null) {
-            throw new NullPointerException("buffer cannot be null");
-         }
          final int propertiesLocation = this.propertiesLocation;
          if (propertiesLocation < 0) {
             throw new IllegalStateException("propertiesLocation = " + propertiesLocation);
@@ -1271,9 +1297,17 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    public String toString() {
       try {
          final TypedProperties properties = getProperties();
-         return "CoreMessage[messageID=" + messageID + ",durable=" + isDurable() + ",userID=" + getUserID() + ",priority=" + this.getPriority()  +
-            ", timestamp=" + toDate(getTimestamp()) + ",expiration=" + toDate(getExpiration()) +
-            ", durable=" + durable + ", address=" + getAddress() + ",size=" + getPersistentSize() + ",properties=" + properties + "]@" + System.identityHashCode(this);
+         return "CoreMessage[messageID=" + messageID +
+            ", durable=" + isDurable() +
+            ", userID=" + getUserID() +
+            ", priority=" + this.getPriority() +
+            ", timestamp=" + toDate(getTimestamp()) +
+            ", expiration=" + toDate(getExpiration()) +
+            ", durable=" + durable +
+            ", address=" + getAddress() +
+            ", size=" + getPersistentSize() +
+            ", properties=" + properties +
+            "]@" + System.identityHashCode(this);
       } catch (Throwable e) {
          logger.warn("Error creating String for message: ", e);
          return "ServerMessage[messageID=" + messageID + "]";
@@ -1289,11 +1323,11 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    }
 
    private SimpleString key(String key) {
-      return SimpleString.toSimpleString(key, getPropertyKeysPool());
+      return SimpleString.of(key, getPropertyKeysPool());
    }
 
    private SimpleString value(String value) {
-      return SimpleString.toSimpleString(value, getPropertyValuesPool());
+      return SimpleString.of(value, getPropertyValuesPool());
    }
 
    private SimpleString.StringSimpleStringPool getPropertyKeysPool() {
@@ -1348,18 +1382,17 @@ public class CoreMessage extends RefCountMessage implements ICoreMessage {
    @Override
    public CompositeData toCompositeData(int fieldsLimit, int deliveryCount) throws OpenDataException {
       CompositeType ct;
-      Map<String, Object> fields;
       byte type = getType();
-      switch (type) {
-         case Message.TEXT_TYPE:
+      Map<String, Object> fields = switch (type) {
+         case Message.TEXT_TYPE -> {
             ct = TEXT_FACTORY.getCompositeType();
-            fields = TEXT_FACTORY.getFields(this, fieldsLimit, deliveryCount);
-            break;
-         default:
+            yield TEXT_FACTORY.getFields(this, fieldsLimit, deliveryCount);
+         }
+         default -> {
             ct = BYTES_FACTORY.getCompositeType();
-            fields = BYTES_FACTORY.getFields(this, fieldsLimit, deliveryCount);
-            break;
-      }
+            yield BYTES_FACTORY.getFields(this, fieldsLimit, deliveryCount);
+         }
+      };
       return new CompositeDataSupport(ct, fields);
 
    }
