@@ -21,11 +21,12 @@ import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPF
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.ADDRESS_AUTO_DELETE_DELAY;
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.ADDRESS_AUTO_DELETE_MSG_COUNT;
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_ADDRESS_RECEIVER;
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_POLICY_NAME;
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationPolicySupport.FEDERATED_ADDRESS_SOURCE_PROPERTIES;
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationPolicySupport.MESSAGE_HOPS_PROPERTY;
-import static org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledMessageConstants.AMQP_TUNNELED_CORE_LARGE_MESSAGE_FORMAT;
-import static org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledMessageConstants.AMQP_TUNNELED_CORE_MESSAGE_FORMAT;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.AMQP_LINK_INITIALIZER_KEY;
+import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.verifyDesiredCapability;
+
 import java.lang.invoke.MethodHandles;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -33,20 +34,14 @@ import java.util.Map;
 import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.atomic.AtomicLong;
-import java.util.function.BiConsumer;
-
-import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.ICoreMessage;
 import org.apache.activemq.artemis.api.core.Message;
 import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
-import org.apache.activemq.artemis.core.config.TransformerConfiguration;
-import org.apache.activemq.artemis.core.server.ActiveMQServerLogger;
 import org.apache.activemq.artemis.core.server.AddressQueryResult;
-import org.apache.activemq.artemis.core.server.transformer.Transformer;
 import org.apache.activemq.artemis.core.transaction.Transaction;
 import org.apache.activemq.artemis.protocol.amqp.broker.AMQPMessage;
+import org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationMetrics.ConsumerMetrics;
 import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPException;
 import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPInternalErrorException;
 import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPNotFoundException;
@@ -58,9 +53,6 @@ import org.apache.activemq.artemis.protocol.amqp.proton.AMQPSessionContext;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpJmsSelectorFilter;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpNoLocalFilter;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport;
-import org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledCoreLargeMessageReader;
-import org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledCoreMessageReader;
-import org.apache.activemq.artemis.protocol.amqp.proton.MessageReader;
 import org.apache.activemq.artemis.protocol.amqp.proton.ProtonServerReceiverContext;
 import org.apache.activemq.artemis.reader.MessageUtil;
 import org.apache.qpid.proton.amqp.Symbol;
@@ -77,11 +69,10 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * Consumer implementation for Federated Addresses that receives from a remote
- * AMQP peer and forwards those messages onto the internal broker Address for
- * consumption by an attached consumers.
+ * Consumer implementation for Federated Addresses that receives from a remote AMQP peer and forwards those messages
+ * onto the internal broker Address for consumption by an attached consumers.
  */
-public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
+public final class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
 
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
@@ -89,222 +80,168 @@ public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
    private static final SimpleString MESSAGE_HOPS_ANNOTATION =
       SimpleString.of(AMQPFederationPolicySupport.MESSAGE_HOPS_ANNOTATION.toString());
 
-   // Sequence ID value used to keep links that would otherwise have the same name from overlapping
-   // this generally occurs when a remote link detach is delayed and new demand is added before it
-   // arrives resulting in an unintended link stealing scenario in the proton engine.
-   private static final AtomicLong LINK_SEQUENCE_ID = new AtomicLong();
-
-   private final AMQPFederationAddressPolicyManager manager;
    private final FederationReceiveFromAddressPolicy policy;
-   private final Transformer transformer;
-
-   private AMQPFederatedAddressDeliveryReceiver receiver;
 
    public AMQPFederationAddressConsumer(AMQPFederationAddressPolicyManager manager,
                                         AMQPFederationConsumerConfiguration configuration,
                                         AMQPSessionContext session, FederationConsumerInfo consumerInfo,
-                                        BiConsumer<FederationConsumerInfo, Message> messageObserver) {
-      super(manager.getFederation(), configuration, session, consumerInfo, messageObserver);
+                                        ConsumerMetrics metrics) {
+      super(manager, configuration, session, consumerInfo, manager.getPolicy(), metrics);
 
-      this.manager = manager;
       this.policy = manager.getPolicy();
-
-      final TransformerConfiguration transformerConfiguration = policy.getTransformerConfiguration();
-      if (transformerConfiguration != null) {
-         this.transformer = federation.getServer().getServiceRegistry().getFederationTransformer(policy.getPolicyName(), transformerConfiguration);
-      } else {
-         this.transformer = (m) -> m;
-      }
-   }
-
-   /**
-    * @return the {@link FederationReceiveFromAddressPolicy} that initiated this consumer.
-    */
-   public FederationReceiveFromAddressPolicy getPolicy() {
-      return policy;
-   }
-
-   private void signalBeforeFederationConsumerMessageHandled(Message message) throws ActiveMQException {
-      try {
-         federation.getServer().callBrokerAMQPFederationPlugins((plugin) -> {
-            if (plugin instanceof ActiveMQServerAMQPFederationPlugin) {
-               ((ActiveMQServerAMQPFederationPlugin) plugin).beforeFederationConsumerMessageHandled(this, message);
-            }
-         });
-      } catch (ActiveMQException t) {
-         ActiveMQServerLogger.LOGGER.federationPluginExecutionError("beforeFederationConsumerMessageHandled", t);
-      }
-   }
-
-   private void signalAfterFederationConsumerMessageHandled(Message message) throws ActiveMQException {
-      try {
-         federation.getServer().callBrokerAMQPFederationPlugins((plugin) -> {
-            if (plugin instanceof ActiveMQServerAMQPFederationPlugin) {
-               ((ActiveMQServerAMQPFederationPlugin) plugin).afterFederationConsumerMessageHandled(this, message);
-            }
-         });
-      } catch (ActiveMQException t) {
-         ActiveMQServerLogger.LOGGER.federationPluginExecutionError("afterFederationConsumerMessageHandled", t);
-      }
    }
 
    private String generateLinkName() {
-      return "federation-" + federation.getName() +
-             "-address-receiver-" + consumerInfo.getAddress() +
-             "-" + federation.getServer().getNodeID() +
-             "-" + LINK_SEQUENCE_ID.incrementAndGet();
+      if (federation.getCapabilities().isUseFQQNAddressSubscriptions()) {
+         return "federation-" + federation.getName() +
+                "-policy-" + policy.getPolicyName() +
+                "-address-receiver-" + consumerInfo.getAddress() +
+                "-" + federation.getServer().getNodeID() +
+                "-" + LINK_SEQUENCE_ID.incrementAndGet();
+      } else {
+         // Stable legacy link naming that is used as the address subscription. This uses the
+         // non-sequence ID version which can allow link stealing to grab a closing link which
+         // will eventually result in the broker connection being closed and rebuilt as we see
+         // it as an unexpected link detach but this allows for stable names and eventually
+         // connection recovery which is arguably less broken than the sequence ID variant which
+         // creates unstable subscription queues that can be orphaned or consumed out of order.
+         return "federation-" + federation.getName() +
+                "-address-receiver-" + consumerInfo.getAddress() +
+                "-" + federation.getServer().getNodeID();
+      }
    }
 
    @Override
-   protected final void asyncCreateReceiver() {
-      connection.runLater(() -> {
-         if (closed) {
-            return;
+   public int getReceiverIdleTimeout() {
+      return configuration.getAddressReceiverIdleTimeout();
+   }
+
+   @Override
+   protected void doCreateReceiver() {
+      try {
+         final Receiver protonReceiver = session.getSession().receiver(generateLinkName());
+         final Target target = new Target();
+         final Source source = new Source();
+
+         if (RoutingType.ANYCAST.equals(consumerInfo.getRoutingType())) {
+            source.setCapabilities(AmqpSupport.QUEUE_CAPABILITY);
+         } else {
+            source.setCapabilities(AmqpSupport.TOPIC_CAPABILITY);
          }
 
-         try {
-            final Receiver protonReceiver = session.getSession().receiver(generateLinkName());
-            final Target target = new Target();
-            final Source source = new Source();
-            final String address = consumerInfo.getAddress();
+         final Map<Symbol, Object> filtersMap = new HashMap<>();
+         filtersMap.put(AmqpSupport.NO_LOCAL_NAME, AmqpNoLocalFilter.NO_LOCAL);
 
-            if (RoutingType.ANYCAST.equals(consumerInfo.getRoutingType())) {
-               source.setCapabilities(AmqpSupport.QUEUE_CAPABILITY);
-            } else {
-               source.setCapabilities(AmqpSupport.TOPIC_CAPABILITY);
-            }
+         if (consumerInfo.getFilterString() != null && !consumerInfo.getFilterString().isEmpty()) {
+            final AmqpJmsSelectorFilter jmsFilter = new AmqpJmsSelectorFilter(consumerInfo.getFilterString());
 
-            final Map<Symbol, Object> filtersMap = new HashMap<>();
-            filtersMap.put(AmqpSupport.NO_LOCAL_NAME, AmqpNoLocalFilter.NO_LOCAL);
+            filtersMap.put(AmqpSupport.JMS_SELECTOR_KEY, jmsFilter);
+         }
 
-            if (consumerInfo.getFilterString() != null && !consumerInfo.getFilterString().isEmpty()) {
-               final AmqpJmsSelectorFilter jmsFilter = new AmqpJmsSelectorFilter(consumerInfo.getFilterString());
+         source.setOutcomes(Arrays.copyOf(OUTCOMES, OUTCOMES.length));
+         source.setDefaultOutcome(DEFAULT_OUTCOME);
+         source.setDurable(TerminusDurability.NONE);
+         source.setExpiryPolicy(TerminusExpiryPolicy.LINK_DETACH);
+         source.setFilter(filtersMap);
 
-               filtersMap.put(AmqpSupport.JMS_SELECTOR_KEY, jmsFilter);
-            }
+         if (federation.getCapabilities().isUseFQQNAddressSubscriptions()) {
+            source.setAddress(consumerInfo.getFqqn());
+         } else {
+            source.setAddress(consumerInfo.getAddress()); // Legacy behavior
+         }
 
-            source.setOutcomes(Arrays.copyOf(OUTCOMES, OUTCOMES.length));
-            source.setDefaultOutcome(DEFAULT_OUTCOME);
-            source.setDurable(TerminusDurability.NONE);
-            source.setExpiryPolicy(TerminusExpiryPolicy.LINK_DETACH);
-            source.setAddress(address);
-            source.setFilter(filtersMap);
+         target.setAddress(consumerInfo.getAddress());
 
-            target.setAddress(address);
+         final Map<String, Object> addressSourceProperties = new HashMap<>();
+         // If the remote needs to create the address then it should apply these
+         // settings during the create.
+         addressSourceProperties.put(ADDRESS_AUTO_DELETE, policy.isAutoDelete());
+         addressSourceProperties.put(ADDRESS_AUTO_DELETE_DELAY, policy.getAutoDeleteDelay());
+         addressSourceProperties.put(ADDRESS_AUTO_DELETE_MSG_COUNT, policy.getAutoDeleteMessageCount());
 
-            final Map<String, Object> addressSourceProperties = new HashMap<>();
-            // If the remote needs to create the address then it should apply these
-            // settings during the create.
-            addressSourceProperties.put(ADDRESS_AUTO_DELETE, policy.isAutoDelete());
-            addressSourceProperties.put(ADDRESS_AUTO_DELETE_DELAY, policy.getAutoDeleteDelay());
-            addressSourceProperties.put(ADDRESS_AUTO_DELETE_MSG_COUNT, policy.getAutoDeleteMessageCount());
+         final Map<Symbol, Object> receiverProperties = new HashMap<>();
+         receiverProperties.put(FEDERATED_ADDRESS_SOURCE_PROPERTIES, addressSourceProperties);
+         receiverProperties.put(FEDERATION_POLICY_NAME, policy.getPolicyName());
 
-            final Map<Symbol, Object> receiverProperties = new HashMap<>();
-            receiverProperties.put(FEDERATED_ADDRESS_SOURCE_PROPERTIES, addressSourceProperties);
+         protonReceiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
+         protonReceiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
+         protonReceiver.setDesiredCapabilities(new Symbol[] {FEDERATION_ADDRESS_RECEIVER});
+         // If enabled offer core tunneling which we prefer to AMQP conversions of core as
+         // the large ones will be converted to standard AMQP messages in memory. When not
+         // offered the remote must not use core tunneling and AMQP conversion will be the
+         // fallback.
+         if (configuration.isCoreMessageTunnelingEnabled()) {
+            protonReceiver.setOfferedCapabilities(new Symbol[] {AmqpSupport.CORE_MESSAGE_TUNNELING_SUPPORT});
+         }
+         protonReceiver.setProperties(receiverProperties);
+         protonReceiver.setTarget(target);
+         protonReceiver.setSource(source);
+         protonReceiver.open();
 
-            protonReceiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
-            protonReceiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
-            protonReceiver.setDesiredCapabilities(new Symbol[] {FEDERATION_ADDRESS_RECEIVER});
-            // If enabled offer core tunneling which we prefer to AMQP conversions of core as
-            // the large ones will be converted to standard AMQP messages in memory. When not
-            // offered the remote must not use core tunneling and AMQP conversion will be the
-            // fallback.
-            if (configuration.isCoreMessageTunnelingEnabled()) {
-               protonReceiver.setOfferedCapabilities(new Symbol[] {AmqpSupport.CORE_MESSAGE_TUNNELING_SUPPORT});
-            }
-            protonReceiver.setProperties(receiverProperties);
-            protonReceiver.setTarget(target);
-            protonReceiver.setSource(source);
-            protonReceiver.open();
+         final ScheduledFuture<?> openTimeoutTask;
+         final AtomicBoolean openTimedOut = new AtomicBoolean(false);
 
-            final ScheduledFuture<?> openTimeoutTask;
-            final AtomicBoolean openTimedOut = new AtomicBoolean(false);
+         if (configuration.getLinkAttachTimeout() > 0) {
+            openTimeoutTask = federation.getServer().getScheduledPool().schedule(() -> {
+               openTimedOut.set(true);
+               federation.signalResourceCreateError(ActiveMQAMQPProtocolMessageBundle.BUNDLE.brokerConnectionTimeout());
+            }, configuration.getLinkAttachTimeout(), TimeUnit.SECONDS);
+         } else {
+            openTimeoutTask = null;
+         }
 
-            if (configuration.getLinkAttachTimeout() > 0) {
-               openTimeoutTask = federation.getServer().getScheduledPool().schedule(() -> {
-                  openTimedOut.set(true);
-                  federation.signalResourceCreateError(ActiveMQAMQPProtocolMessageBundle.BUNDLE.brokerConnectionTimeout());
-               }, configuration.getLinkAttachTimeout(), TimeUnit.SECONDS);
-            } else {
-               openTimeoutTask = null;
-            }
+         this.protonReceiver = protonReceiver;
 
-            this.protonReceiver = protonReceiver;
-
-            protonReceiver.attachments().set(AMQP_LINK_INITIALIZER_KEY, Runnable.class, () -> {
-               try {
-                  if (openTimeoutTask != null) {
-                     openTimeoutTask.cancel(false);
-                  }
-
-                  if (openTimedOut.get()) {
-                     return;
-                  }
-
-                  // Remote must support federation receivers otherwise we fail the connection unless the
-                  // Attach indicates that a detach is incoming in which case we just allow the normal handling
-                  // to occur.
-                  if (protonReceiver.getRemoteSource() != null && !AmqpSupport.verifyOfferedCapabilities(protonReceiver, FEDERATION_ADDRESS_RECEIVER)) {
-                     federation.signalResourceCreateError(
-                        ActiveMQAMQPProtocolMessageBundle.BUNDLE.missingOfferedCapability(FEDERATION_ADDRESS_RECEIVER.toString()));
-                     return;
-                  }
-
-                  // Intercept remote close and check for valid reasons for remote closure such as
-                  // the remote peer not having a matching queue for this subscription or from an
-                  // operator manually closing the link.
-                  federation.addLinkClosedInterceptor(consumerInfo.getId(), remoteCloseInterceptor);
-
-                  receiver = new AMQPFederatedAddressDeliveryReceiver(session, consumerInfo, protonReceiver);
-
-                  if (protonReceiver.getRemoteSource() != null) {
-                     logger.debug("AMQP Federation {} address consumer {} completed open", federation.getName(), consumerInfo);
-                  } else {
-                     logger.debug("AMQP Federation {} address consumer {} rejected by remote", federation.getName(), consumerInfo);
-                  }
-
-                  session.addReceiver(protonReceiver, (session, protonRcvr) -> {
-                     return this.receiver;
-                  });
-               } catch (Exception e) {
-                  federation.signalError(e);
+         protonReceiver.attachments().set(AMQP_LINK_INITIALIZER_KEY, Runnable.class, () -> {
+            try {
+               if (openTimeoutTask != null) {
+                  openTimeoutTask.cancel(false);
                }
-            });
-         } catch (Exception e) {
-            federation.signalError(e);
-         }
 
-         connection.flush();
-      });
-   }
+               if (openTimedOut.get()) {
+                  return;
+               }
 
-   @Override
-   protected final void asyncCloseReceiver() {
-      connection.runLater(() -> {
-         federation.removeLinkClosedInterceptor(consumerInfo.getId());
+               // Remote must support federation receivers otherwise we fail the connection unless the
+               // Attach indicates that a detach is incoming in which case we just allow the normal handling
+               // to occur.
+               if (protonReceiver.getRemoteSource() != null && !AmqpSupport.verifyOfferedCapabilities(protonReceiver, FEDERATION_ADDRESS_RECEIVER)) {
+                  federation.signalResourceCreateError(
+                     ActiveMQAMQPProtocolMessageBundle.BUNDLE.missingOfferedCapability(FEDERATION_ADDRESS_RECEIVER.toString()));
+                  return;
+               }
 
-         if (receiver != null) {
-            try {
-               receiver.close(false);
-            } catch (ActiveMQAMQPException e) {
-            } finally {
-               receiver = null;
+               // Intercept remote close and check for valid reasons for remote closure such as
+               // the remote peer not having a matching queue for this subscription or from an
+               // operator manually closing the link.
+               federation.addLinkClosedInterceptor(consumerInfo.getId(), remoteCloseInterceptor);
+
+               receiver = new AMQPFederatedAddressDeliveryReceiver(session, consumerInfo, protonReceiver);
+
+               final boolean linkOpened = protonReceiver.getRemoteSource() != null;
+
+               if (linkOpened) {
+                  logger.debug("AMQP Federation {} address consumer {} completed open", federation.getName(), consumerInfo);
+               } else {
+                  logger.debug("AMQP Federation {} address consumer {} rejected by remote", federation.getName(), consumerInfo);
+               }
+
+               session.addReceiver(protonReceiver, (session, protonRcvr) -> {
+                  return this.receiver;
+               });
+
+               if (linkOpened && remoteOpenHandler != null) {
+                  remoteOpenHandler.accept(this);
+               }
+            } catch (Exception e) {
+               federation.signalError(e);
             }
-         }
+         });
+      } catch (Exception e) {
+         federation.signalError(e);
+      }
 
-         // Need to track the proton receiver and close it here as the default
-         // context implementation doesn't do that and could result in no detach
-         // being sent in some cases and possible resources leaks.
-         if (protonReceiver != null) {
-            try {
-               protonReceiver.close();
-            } finally {
-               protonReceiver = null;
-            }
-         }
-
-         connection.flush();
-      });
+      connection.flush();
    }
 
    private static AMQPMessage incrementAMQPMessageHops(AMQPMessage message) {
@@ -337,24 +274,20 @@ public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
    }
 
    /**
-    * Wrapper around the standard receiver context that provides federation specific entry
-    * points and customizes inbound delivery handling for this Address receiver.
+    * Wrapper around the standard receiver context that provides federation specific entry points and customizes inbound
+    * delivery handling for this Address receiver.
     */
    private class AMQPFederatedAddressDeliveryReceiver extends ProtonServerReceiverContext {
 
       private final SimpleString cachedAddress;
 
-      private MessageReader coreMessageReader;
-      private MessageReader coreLargeMessageReader;
       private boolean closed;
 
       /**
        * Creates the federation receiver instance.
        *
-       * @param session
-       *    The server session context bound to the receiver instance.
-       * @param receiver
-       *    The proton receiver that will be wrapped in this server context instance.
+       * @param session  The server session context bound to the receiver instance.
+       * @param receiver The proton receiver that will be wrapped in this server context instance.
        */
       AMQPFederatedAddressDeliveryReceiver(AMQPSessionContext session, FederationConsumerInfo consumerInfo, Receiver receiver) {
          super(session.getSessionSPI(), session.getAMQPConnectionContext(), session, receiver);
@@ -370,7 +303,7 @@ public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
             closed = true;
 
             try {
-               federation.unregisterAddressConsumerManagement(manager, AMQPFederationAddressConsumer.this);
+               federation.unregisterFederationConsumerManagement(AMQPFederationAddressConsumer.this);
             } catch (Exception e) {
                logger.trace("Error thrown when unregistering federation address consumer from management", e);
             }
@@ -440,26 +373,21 @@ public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
             throw new ActiveMQAMQPInternalErrorException(e.getMessage(), e);
          }
 
+         if (configuration.isCoreMessageTunnelingEnabled()) {
+            // We will have offered it if the option is enabled, but the remote needs to indicate it desires it
+            // otherwise we want to fail on any tunneled core messages that arrives which is the default.
+            if (verifyDesiredCapability(protonReceiver, AmqpSupport.CORE_MESSAGE_TUNNELING_SUPPORT)) {
+               enableCoreTunneling();
+            }
+         }
+
          try {
-            federation.registerAddressConsumerManagement(manager, AMQPFederationAddressConsumer.this);
+            federation.registerFederationConsumerManagement(AMQPFederationAddressConsumer.this);
          } catch (Exception e) {
             logger.debug("Error caught when trying to add federation address consumer to management", e);
          }
 
-         flow();
-      }
-
-      @Override
-      protected MessageReader trySelectMessageReader(Receiver receiver, Delivery delivery) {
-         if (delivery.getMessageFormat() == AMQP_TUNNELED_CORE_MESSAGE_FORMAT) {
-            return coreMessageReader != null ?
-               coreMessageReader : (coreMessageReader = new AMQPTunneledCoreMessageReader(this));
-         } else if (delivery.getMessageFormat() == AMQP_TUNNELED_CORE_LARGE_MESSAGE_FORMAT) {
-            return coreLargeMessageReader != null ?
-               coreLargeMessageReader : (coreLargeMessageReader = new AMQPTunneledCoreLargeMessageReader(this));
-         } else {
-            return super.trySelectMessageReader(receiver, delivery);
-         }
+         topUpCreditIfNeeded();
       }
 
       @Override
@@ -472,8 +400,8 @@ public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
 
             final Message baseMessage;
 
-            if (message instanceof ICoreMessage) {
-               baseMessage = incrementCoreMessageHops((ICoreMessage) message);
+            if (message instanceof ICoreMessage coreMessage) {
+               baseMessage = incrementCoreMessageHops(coreMessage);
 
                // Add / Update the connection Id value to reflect the remote container Id so that the
                // no-local filter of a federation address receiver directed back to the source of this
@@ -490,9 +418,9 @@ public class AMQPFederationAddressConsumer extends AMQPFederationConsumer {
                             transformer, baseMessage, theMessage);
             }
 
-            signalBeforeFederationConsumerMessageHandled(theMessage);
+            signalPluginBeforeFederationConsumerMessageHandled(theMessage);
             sessionSPI.serverSend(this, tx, receiver, delivery, cachedAddress, routingContext, theMessage);
-            signalAfterFederationConsumerMessageHandled(theMessage);
+            signalPluginAfterFederationConsumerMessageHandled(theMessage);
          } catch (Exception e) {
             logger.warn("Inbound delivery for {} encountered an error: {}", consumerInfo, e.getMessage(), e);
             deliveryFailed(delivery, receiver, e);
