@@ -18,11 +18,17 @@
 package org.apache.activemq.artemis.protocol.amqp.connect.mirror;
 
 import java.lang.invoke.MethodHandles;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
+import java.util.concurrent.RejectedExecutionException;
+import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.LongSupplier;
 
 import io.netty.util.collection.LongObjectHashMap;
@@ -31,7 +37,6 @@ import org.apache.activemq.artemis.api.core.QueueConfiguration;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.core.config.Configuration;
 import org.apache.activemq.artemis.core.io.IOCriticalErrorListener;
-import org.apache.activemq.artemis.core.journal.Journal;
 import org.apache.activemq.artemis.core.journal.RecordInfo;
 import org.apache.activemq.artemis.core.journal.collections.JournalHashMap;
 import org.apache.activemq.artemis.core.journal.collections.JournalHashMapProvider;
@@ -51,8 +56,10 @@ import org.apache.activemq.artemis.core.server.RoutingContext;
 import org.apache.activemq.artemis.core.server.impl.AckReason;
 import org.apache.activemq.artemis.core.server.impl.AddressInfo;
 import org.apache.activemq.artemis.core.server.mirror.MirrorController;
+import org.apache.activemq.artemis.core.server.mirror.MirrorRegistry;
 import org.apache.activemq.artemis.core.transaction.Transaction;
 import org.apache.activemq.artemis.core.transaction.impl.TransactionImpl;
+import org.apache.activemq.artemis.protocol.amqp.logger.ActiveMQAMQPProtocolLogger;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -63,7 +70,7 @@ public class AckManager implements ActiveMQComponent {
 
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
-   final Journal journal;
+   final Set<AMQPMirrorControllerTarget> mirrorControllerTargets = new HashSet<>();
    final LongSupplier sequenceGenerator;
    final JournalHashMapProvider<AckRetry, AckRetry, Queue> journalHashMapProvider;
    final ActiveMQServer server;
@@ -73,18 +80,27 @@ public class AckManager implements ActiveMQComponent {
    volatile MultiStepProgress progress;
    ActiveMQScheduledComponent scheduledComponent;
 
+   final MirrorRegistry mirrorRegistry;
+
+   public int size() {
+      return mirrorRegistry.getMirrorAckSize();
+   }
+
    public AckManager(ActiveMQServer server) {
+      assert server != null && server.getConfiguration() != null;
       this.server = server;
       this.configuration = server.getConfiguration();
       this.ioCriticalErrorListener = server.getIoCriticalErrorListener();
-      this.journal = server.getStorageManager().getMessageJournal();
       this.sequenceGenerator = server.getStorageManager()::generateID;
-      journalHashMapProvider = new JournalHashMapProvider<>(sequenceGenerator, journal, AckRetry.getPersister(), JournalRecordIds.ACK_RETRY, OperationContextImpl::getContext, server.getPostOffice()::findQueue, server.getIoCriticalErrorListener());
+      this.mirrorRegistry = server.getMirrorRegistry();
+      // The JournalHashMap has to use the storage manager to guarantee we are using the Replicated Journal Wrapper in case this is a replicated journal
+      journalHashMapProvider = new JournalHashMapProvider<>(sequenceGenerator, server.getStorageManager(), AckRetry.getPersister(), JournalRecordIds.ACK_RETRY, OperationContextImpl::getContext, server.getPostOffice()::findQueue, server.getIoCriticalErrorListener());
       this.referenceIDSupplier = new ReferenceIDSupplier(server);
    }
 
    public void reload(RecordInfo recordInfo) {
       journalHashMapProvider.reload(recordInfo);
+      mirrorRegistry.incrementMirrorAckSize();
    }
 
    @Override
@@ -94,7 +110,14 @@ public class AckManager implements ActiveMQComponent {
          scheduledComponent = null;
       }
       AckManagerProvider.remove(this.server);
-      logger.debug("Stopping ackmanager on server {}", server);
+      logger.trace("Stopping ackmanager on server {}", server);
+   }
+
+   public synchronized void pause() {
+      if (scheduledComponent != null) {
+         scheduledComponent.stop();
+         scheduledComponent = null;
+      }
    }
 
    @Override
@@ -104,8 +127,8 @@ public class AckManager implements ActiveMQComponent {
 
    @Override
    public synchronized void start() {
-      if (logger.isDebugEnabled()) {
-         logger.debug("Starting ACKManager on {} with period = {}, minQueueAttempts={}, maxPageAttempts={}", server, configuration.getMirrorAckManagerRetryDelay(), configuration.getMirrorAckManagerQueueAttempts(), configuration.getMirrorAckManagerPageAttempts());
+      if (logger.isTraceEnabled()) {
+         logger.trace("Starting ACKManager on {} with period = {}, minQueueAttempts={}, maxPageAttempts={}", server, configuration.getMirrorAckManagerRetryDelay(), configuration.getMirrorAckManagerQueueAttempts(), configuration.getMirrorAckManagerPageAttempts());
       }
       if (!isStarted()) {
          scheduledComponent = new ActiveMQScheduledComponent(server.getScheduledPool(), server.getExecutorFactory().getExecutor(), server.getConfiguration().getMirrorAckManagerRetryDelay(), server.getConfiguration().getMirrorAckManagerRetryDelay(), TimeUnit.MILLISECONDS, true) {
@@ -117,7 +140,7 @@ public class AckManager implements ActiveMQComponent {
          scheduledComponent.start();
          scheduledComponent.delay();
       } else {
-         logger.debug("Starting ignored on server {}", server);
+         logger.trace("Starting ignored on server {}", server);
       }
    }
 
@@ -137,7 +160,14 @@ public class AckManager implements ActiveMQComponent {
 
       // schedule a retry
       if (!sortRetries().isEmpty()) {
-         scheduledComponent.delay();
+         ActiveMQScheduledComponent scheduleComponentReference = scheduledComponent;
+         if (scheduleComponentReference != null) {
+            try {
+               scheduleComponentReference.delay();
+            } catch (RejectedExecutionException ree) {
+               logger.debug("AckManager could not schedule a new retry due to the executor being shutdown {}", ree.getMessage(), ree);
+            }
+         }
       }
    }
 
@@ -147,24 +177,49 @@ public class AckManager implements ActiveMQComponent {
          return false;
       }
 
-      HashMap<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retries = sortRetries();
+      Map<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retries = sortRetries();
+
+      flushMirrorTargets();
 
       if (retries.isEmpty()) {
          logger.trace("Nothing to retry!, server={}", server);
          return false;
       }
 
-      progress = new MultiStepProgress(sortRetries());
+      progress = new MultiStepProgress(retries);
       return true;
    }
 
+   public synchronized void registerMirror(AMQPMirrorControllerTarget mirrorTarget) {
+      this.mirrorControllerTargets.add(mirrorTarget);
+   }
+
+   public synchronized void unregisterMirror(AMQPMirrorControllerTarget mirrorTarget) {
+      this.mirrorControllerTargets.remove(mirrorTarget);
+   }
+
+   private void flushMirrorTargets() {
+      logger.trace("scanning and flushing mirror targets");
+      List<AMQPMirrorControllerTarget> targetCopy = copyTargets();
+      targetCopy.forEach(AMQPMirrorControllerTarget::flush);
+   }
+
+   private void checkFlowControlMirrorTargets() {
+      List<AMQPMirrorControllerTarget> targetCopy = copyTargets();
+      targetCopy.forEach(AMQPMirrorControllerTarget::verifyCredits);
+   }
+
+   private synchronized List<AMQPMirrorControllerTarget> copyTargets() {
+      return new ArrayList<>(mirrorControllerTargets);
+   }
+
    // Sort the ACK list by address
    // We have the retries by queue, we need to sort them by address
    // as we will perform all the retries on the same addresses at the same time (in the Multicast case with multiple queues acking)
-   public HashMap<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> sortRetries() {
+   public Map<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> sortRetries() {
       // We will group the retries by address,
       // so we perform all of the queues in the same address at once
-      HashMap<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retriesByAddress = new HashMap<>();
+      Map<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retriesByAddress = new HashMap<>();
 
       Iterator<JournalHashMap<AckRetry, AckRetry, Queue>> queueRetriesIterator = journalHashMapProvider.getMaps().iterator();
 
@@ -187,52 +242,86 @@ public class AckManager implements ActiveMQComponent {
       return retriesByAddress;
    }
 
-   private boolean isEmpty(LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry) {
-      AtomicBoolean empty = new AtomicBoolean(true);
-
-      queuesToRetry.forEach((id, journalHashMap) -> {
-         if (!journalHashMap.isEmpty()) {
-            empty.set(false);
+   private boolean isSnapshotComplete(LongObjectHashMap<AtomicInteger> pendingSnapshot) {
+      AtomicBoolean complete = new AtomicBoolean(true);
+      pendingSnapshot.forEach((l, count) -> {
+         if (count.get() > 0) {
+            complete.set(false);
          }
       });
 
-      return empty.get();
+      return complete.get();
    }
 
-
-
    // to be used with the same executor as the PagingStore executor
    public void retryAddress(SimpleString address, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> acksToRetry) {
+
+
+      // This is an optimization:
+      // we peek at how many records we currently have. When we scan all the records that were initially input we would
+      // interrupt the scan and start over
+      // to avoid scanning for records that will never match since new ones are probably "in the past" now
+      LongObjectHashMap<AtomicInteger> snapshotCount = buildCounterSnapshot(acksToRetry);
+
       MirrorController previousController = AMQPMirrorControllerTarget.getControllerInUse();
       logger.trace("retrying address {} on server {}", address, server);
       try {
          AMQPMirrorControllerTarget.setControllerInUse(disabledAckMirrorController);
 
-         if (checkRetriesAndPaging(acksToRetry)) {
+         if (checkRetriesAndPaging(acksToRetry, snapshotCount)) {
             logger.trace("scanning paging for {}", address);
-            AckRetry key = new AckRetry();
 
             PagingStore store = server.getPagingManager().getPageStore(address);
             for (long pageId = store.getFirstPage(); pageId <= store.getCurrentWritingPage(); pageId++) {
-               if (isEmpty(acksToRetry)) {
-                  logger.trace("Retry stopped while reading page {} on address {} as the outcome is now empty, server={}", pageId, address, server);
+               if (isSnapshotComplete(snapshotCount)) {
+                  logger.debug("AckManager Page Scanning complete (done) on address {}", address);
                   break;
                }
                Page page = openPage(store, pageId);
                if (page == null) {
                   continue;
                }
+
+               if (logger.isDebugEnabled()) {
+                  logger.debug("scanning for acks on page {}/{} on address {}", page.getPageId(), store.getCurrentWritingPage(), address);
+               }
                try {
-                  retryPage(acksToRetry, address, page, key);
+                  retryPage(snapshotCount, acksToRetry, address, page);
                } finally {
                   page.usageDown();
                }
             }
-            validateExpiredSet(acksToRetry);
+
+            // a note on the following statement:
+            // I used to store the result of isSnapshotComplete in a variable
+            // however in the case where we got to the end of the list and at the same time the snapshot was emptied,
+            // we would still print any logging on the next block, and verify the expired sets
+            // You can see the difference between this and the previous commit on ARTEMIS-5564
+            if (!isSnapshotComplete(snapshotCount)) {
+               // isSnapshotComplete == true, it means that every record meant to be acked was used,
+               // so there is no need to check the expired set and we bypass this check
+               // we used to check this every page scan, but as an optimization we removed this unecessary step
+               validateExpiredSet(address, acksToRetry);
+
+               if (logger.isDebugEnabled()) {
+                  logger.debug("Retry page address got to the end of the list without still finding a few records to acknowledge");
+                  snapshotCount.forEach((l, c) -> logger.warn("queue {} still have {} ack records after the scan is finished", l, c));
+                  acksToRetry.forEach((l, m) -> {
+                     logger.debug("Records on queue {}:", l);
+                     m.forEach((ack1, ack2) -> {
+                        if (ack1.getViewCount() > 0) {
+                           logger.debug("Record {}", ack1);
+                        }
+                     });
+                  });
+               }
+            }
          } else {
             logger.trace("Page Scan not required for address {}", address);
          }
 
+         checkFlowControlMirrorTargets();
+
       } catch (Throwable e) {
          logger.warn(e.getMessage(), e);
       } finally {
@@ -240,6 +329,20 @@ public class AckManager implements ActiveMQComponent {
       }
    }
 
+   private static LongObjectHashMap<AtomicInteger> buildCounterSnapshot(LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> acksToRetry) {
+      LongObjectHashMap<AtomicInteger> snapshotCount = new LongObjectHashMap<>();
+      acksToRetry.forEach((l, map) -> {
+         AtomicInteger recordCount = new AtomicInteger(0);
+         snapshotCount.put(l, recordCount);
+         map.forEach((ack1, ack2) -> {
+            ack2.incrementViewCount();
+            recordCount.incrementAndGet();
+         });
+         logger.trace("Building counter snapshot for Queue {} with {} ack elements", l, recordCount);
+      });
+      return snapshotCount;
+   }
+
    private Page openPage(PagingStore store, long pageID) throws Throwable {
       Page page = store.newPageObject(pageID);
       if (page.getFile().exists()) {
@@ -251,38 +354,49 @@ public class AckManager implements ActiveMQComponent {
 
    }
 
-   private void validateExpiredSet(LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry) {
-      queuesToRetry.forEach(this::validateExpireSet);
+   private void validateExpiredSet(SimpleString address, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry) {
+      queuesToRetry.forEach((q, r) -> this.validateExpireSet(address, q, r));
    }
 
-   private void validateExpireSet(long queueID, JournalHashMap<AckRetry, AckRetry, Queue> retries) {
+   private void validateExpireSet(SimpleString address, long queueID, JournalHashMap<AckRetry, AckRetry, Queue> retries) {
       for (AckRetry retry : retries.valuesCopy()) {
-         if (retry.getQueueAttempts() >= configuration.getMirrorAckManagerQueueAttempts()) {
+         // we only remove or configure to be removed if the retry was initially seen on the start of the process
+         // this is to avoid a race where an ACK entered the list after the scan been through where the element was supposed to be
+         if (retry.getViewCount() > 0 && retry.getQueueAttempts() >= configuration.getMirrorAckManagerQueueAttempts()) {
             if (retry.attemptedPage() >= configuration.getMirrorAckManagerPageAttempts()) {
-               if (logger.isDebugEnabled()) {
-                  logger.debug("Retried {} {} times, giving up on the entry now", retry, retry.getPageAttempts());
+               if (configuration.isMirrorAckManagerWarnUnacked()) {
+                  ActiveMQAMQPProtocolLogger.LOGGER.ackRetryFailed(retry, address, queueID);
                }
-               retries.remove(retry);
-            } else {
                if (logger.isDebugEnabled()) {
-                  logger.trace("Retry {} attempted {} times on paging", retry, retry.getPageAttempts());
+                  logger.debug("Retried {} {} times, giving up on the entry now. Configured Page Attempts={}", retry, retry.getPageAttempts(), configuration.getMirrorAckManagerPageAttempts());
+               }
+               if (retries.remove(retry) != null) {
+                  mirrorRegistry.decrementMirrorAckSize();
+               }
+            } else {
+               if (logger.isTraceEnabled()) {
+                  logger.trace("Retry {} attempted {} times on paging, Configuration Page Attempts={}", retry, retry.getPageAttempts(), configuration.getMirrorAckManagerPageAttempts());
                }
             }
+         } else {
+            logger.trace("Retry {} queue attempted {} times on paging, QueueAttempts {} Configuration Page Attempts={}", retry, retry.getQueueAttempts(), retry.getPageAttempts(), configuration.getMirrorAckManagerPageAttempts());
          }
       }
    }
 
-   private void retryPage(LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry,
+   private void retryPage(LongObjectHashMap<AtomicInteger> snapshotCount,
+                          LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry,
                           SimpleString address,
-                          Page page,
-                          AckRetry key) throws Exception {
-      logger.debug("scanning for acks on page {} on address {}", page.getPageId(), address);
+                          Page page) throws Exception {
+
+      AckRetry key = new AckRetry();
       TransactionImpl transaction = new TransactionImpl(server.getStorageManager()).setAsync(true);
       // scan each page for acks
       page.getMessages().forEach(pagedMessage -> {
          for (int i = 0; i < pagedMessage.getQueueIDs().length; i++) {
             long queueID = pagedMessage.getQueueIDs()[i];
             JournalHashMap<AckRetry, AckRetry, Queue> retries = queuesToRetry.get(queueID);
+            AtomicInteger snapshotOnQueue = snapshotCount.get(queueID);
             if (retries != null) {
                String serverID = referenceIDSupplier.getServerID(pagedMessage.getMessage());
                if (serverID == null) {
@@ -290,14 +404,11 @@ public class AckManager implements ActiveMQComponent {
                }
                long id = referenceIDSupplier.getID(pagedMessage.getMessage());
 
-               logger.trace("Looking for retry on serverID={}, id={} on server={}", serverID, id, server);
                key.setNodeID(serverID).setMessageID(id);
 
                AckRetry ackRetry = retries.get(key);
 
-               // we first retry messages in the queue first.
-               // this is to avoid messages that are in transit from being depaged into the queue
-               if (ackRetry != null && ackRetry.getQueueAttempts() > configuration.getMirrorAckManagerQueueAttempts()) {
+               if (ackRetry != null) {
                   Queue queue = retries.getContext();
 
                   if (queue != null) {
@@ -314,13 +425,14 @@ public class AckManager implements ActiveMQComponent {
                            }
                         }
                      }
-                     retries.remove(ackRetry, transaction.getID());
+                     if (retries.remove(ackRetry, transaction.getID()) != null) {
+                        mirrorRegistry.decrementMirrorAckSize();
+                        decrementSnapshotCount(ackRetry, snapshotOnQueue);
+                     }
                      transaction.setContainsPersistent();
                      logger.trace("retry performed ok, ackRetry={} for message={} on queue", ackRetry, pagedMessage);
                   }
                }
-            } else {
-               logger.trace("Retry key={} not found server={}", key, server);
             }
          }
       });
@@ -335,8 +447,28 @@ public class AckManager implements ActiveMQComponent {
       }
    }
 
-   /** returns true if there are retries ready to be scanned on paging */
-   private boolean checkRetriesAndPaging(LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry) {
+   private void decrementSnapshotCount(AckRetry retry, AtomicInteger queueSnapshotCount) {
+      // we check the view count as we only decrement the snapshot if the record was
+      // in the initial list when we started the scan
+      // otherwise we will of course ack the message since we have the record anyways
+      // but we won't discount it from the snapshot counter
+      if (retry.getViewCount() > 0) {
+         if (queueSnapshotCount != null) {
+            queueSnapshotCount.decrementAndGet();
+         }
+      } else {
+         // we count the initial records on the retry list
+         // however more records could still be flowing while the scan is being performed
+         // on this case the record entered the scan list, we can actually acknowledge the entry, but we should not decrement
+         // the snapshot count as it wasn't part of the initial list
+         logger.trace("Update on snapshot count ignrored for {}", retry);
+      }
+   }
+
+   /**
+    * {@return {@code true} if there are retries ready to be scanned on paging}
+    */
+   private boolean checkRetriesAndPaging(LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>> queuesToRetry, LongObjectHashMap<AtomicInteger> snapshotCount) {
       boolean needScanOnPaging = false;
       Iterator<Map.Entry<Long, JournalHashMap<AckRetry, AckRetry, Queue>>> iter = queuesToRetry.entrySet().iterator();
 
@@ -344,10 +476,13 @@ public class AckManager implements ActiveMQComponent {
          Map.Entry<Long, JournalHashMap<AckRetry, AckRetry, Queue>> entry = iter.next();
          JournalHashMap<AckRetry, AckRetry, Queue> queueRetries = entry.getValue();
          Queue queue = queueRetries.getContext();
+         AtomicInteger queueSnapshotCount = snapshotCount.get(queue.getID());
          for (AckRetry retry : queueRetries.valuesCopy()) {
             if (ack(retry.getNodeID(), queue, retry.getMessageID(), retry.getReason(), false)) {
                logger.trace("Removing retry {} as the retry went ok", retry);
                queueRetries.remove(retry);
+               mirrorRegistry.decrementMirrorAckSize();
+               decrementSnapshotCount(retry, queueSnapshotCount);
             } else {
                int retried = retry.attemptedQueue();
                if (logger.isTraceEnabled()) {
@@ -369,6 +504,7 @@ public class AckManager implements ActiveMQComponent {
       }
       AckRetry retry = new AckRetry(nodeID, messageID, reason);
       journalHashMapProvider.getMap(queue.getID(), queue).put(retry, retry);
+      mirrorRegistry.incrementMirrorAckSize();
       if (scheduledComponent != null) {
          // we set the retry delay again in case it was changed.
          scheduledComponent.setPeriod(configuration.getMirrorAckManagerRetryDelay());
@@ -384,11 +520,16 @@ public class AckManager implements ActiveMQComponent {
       MessageReference reference = targetQueue.removeWithSuppliedID(nodeID, messageID, referenceIDSupplier);
 
       if (reference == null) {
-         if (logger.isDebugEnabled()) {
-            logger.debug("ACK Manager could not find reference nodeID={} (while localID={}), messageID={} on queue {}, server={}. Adding retry with minQueue={}, maxPage={}, delay={}", nodeID, referenceIDSupplier.getDefaultNodeID(), messageID, targetQueue.getName(), server, configuration.getMirrorAckManagerQueueAttempts(), configuration.getMirrorAckManagerPageAttempts(), configuration.getMirrorAckManagerRetryDelay());
-            printQueueDebug(targetQueue);
+         if (logger.isTraceEnabled()) {
+            logger.trace("ACK Manager could not find reference nodeID={} (while localID={}), messageID={} on queue {}, server={}. Adding retry with minQueue={}, maxPage={}, delay={}", nodeID, referenceIDSupplier.getDefaultNodeID(), messageID, targetQueue.getName(), server, configuration.getMirrorAckManagerQueueAttempts(), configuration.getMirrorAckManagerPageAttempts(), configuration.getMirrorAckManagerRetryDelay());
          }
+
          if (allowRetry) {
+            if (configuration != null && configuration.isMirrorAckManagerWarnUnacked() && targetQueue.getConsumerCount() > 0) {
+               ActiveMQAMQPProtocolLogger.LOGGER.unackWithConsumer(targetQueue.getConsumerCount(), targetQueue.getName(), nodeID, messageID);
+            } else {
+               logger.trace("There are {} consumers on queue {}, what made Ack for message with nodeID={}, messageID={} enter a retry list", targetQueue.getConsumerCount(), targetQueue.getName(), nodeID, messageID);
+            }
             addRetry(nodeID, targetQueue, messageID, reason);
          }
          return false;
@@ -404,10 +545,6 @@ public class AckManager implements ActiveMQComponent {
       }
    }
 
-   private void printQueueDebug(Queue targetQueue) {
-      logger.debug("... queue {}/{} had {} consumers, {} messages, {} scheduled messages, {} delivering messages, paging={}", targetQueue.getID(), targetQueue.getName(), targetQueue.getConsumerCount(), targetQueue.getMessageCount(), targetQueue.getScheduledCount(), targetQueue.getDeliveringCount(), targetQueue.getPagingStore().isPaging());
-   }
-
    private void doACK(Queue targetQueue, MessageReference reference, AckReason reason) {
       try {
          switch (reason) {
@@ -429,15 +566,17 @@ public class AckManager implements ActiveMQComponent {
          targetQueue.deliverAsync();
       }
    }
-   /** The ACKManager will perform the retry on each address's pageStore executor.
-    *  it will perform each address individually, one by one. */
+   /*
+    * The ACKManager will perform the retry on each address's pageStore executor.
+    * It will perform each address individually, one by one.
+    */
    class MultiStepProgress {
-      HashMap<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retryList;
+      Map<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retryList;
 
       Iterator<Map.Entry<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>>> retryIterator;
 
 
-      MultiStepProgress(HashMap<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retryList) {
+      MultiStepProgress(Map<SimpleString, LongObjectHashMap<JournalHashMap<AckRetry, AckRetry, Queue>>> retryList) {
          this.retryList = retryList;
          retryIterator = retryList.entrySet().iterator();
       }
