@@ -29,6 +29,7 @@ import io.netty.handler.codec.mqtt.MqttProperties;
 import io.netty.handler.codec.mqtt.MqttPublishMessage;
 import io.netty.handler.codec.mqtt.MqttPublishVariableHeader;
 import io.netty.handler.codec.mqtt.MqttQoS;
+import io.netty.handler.codec.mqtt.MqttTopicSubscription;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffer;
 import org.apache.activemq.artemis.api.core.ActiveMQIllegalStateException;
 import org.apache.activemq.artemis.api.core.ActiveMQSecurityException;
@@ -50,6 +51,7 @@ import org.apache.activemq.artemis.utils.UUIDGenerator;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import java.lang.invoke.MethodHandles;
+import java.util.Objects;
 
 import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.CONTENT_TYPE;
 import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.CORRELATION_DATA;
@@ -60,6 +62,7 @@ import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.SUBSCR
 import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.TOPIC_ALIAS;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.MQTT_CONTENT_TYPE_KEY;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.MQTT_CORRELATION_DATA_KEY;
+import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.MQTT_MESSAGE_RETAIN_INITIAL_DISTRIBUTION_KEY;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.MQTT_MESSAGE_RETAIN_KEY;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.MQTT_PAYLOAD_FORMAT_INDICATOR_KEY;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.MQTT_RESPONSE_TOPIC_KEY;
@@ -128,13 +131,13 @@ public class MQTTPublishManager {
    }
 
    private SimpleString createManagementAddress() {
-      return new SimpleString(MQTTUtil.MANAGEMENT_QUEUE_PREFIX + session.getState().getClientId());
+      return SimpleString.of(MQTTUtil.MANAGEMENT_QUEUE_PREFIX + session.getState().getClientId());
    }
 
    private void createManagementQueue() throws Exception {
       Queue q = session.getServer().locateQueue(managementAddress);
       if (q == null) {
-         session.getServer().createQueue(new QueueConfiguration(managementAddress)
+         session.getServer().createQueue(QueueConfiguration.of(managementAddress)
                                             .setRoutingType(RoutingType.ANYCAST)
                                             .setDurable(MQTTUtil.DURABLE_MESSAGES));
       }
@@ -176,7 +179,6 @@ public class MQTTPublishManager {
     * Sends a message either on behalf of the client or on behalf of the broker (Will Messages)
     *
     * @param internal if true means on behalf of the broker (skips authorisation) and does not return ack.
-    * @throws Exception
     */
    void sendToQueue(MqttPublishMessage message, boolean internal) throws Exception {
       synchronized (lock) {
@@ -187,40 +189,45 @@ public class MQTTPublishManager {
          String topic = message.variableHeader().topicName();
          if (session.getVersion() == MQTTVersion.MQTT_5) {
             Integer alias = MQTTUtil.getProperty(Integer.class, message.variableHeader().properties(), TOPIC_ALIAS);
-            Integer topicAliasMax = session.getProtocolManager().getTopicAliasMaximum();
             if (alias != null) {
+               Integer topicAliasMax = session.getProtocolManager().getTopicAliasMaximum();
                if (alias == 0) {
                   // [MQTT-3.3.2-8]
                   throw new DisconnectException(MQTTReasonCodes.TOPIC_ALIAS_INVALID);
                } else if (topicAliasMax != null && alias > topicAliasMax) {
                   // [MQTT-3.3.2-9]
                   throw new DisconnectException(MQTTReasonCodes.TOPIC_ALIAS_INVALID);
-               } else {
-                  topic = session.getState().getClientTopicAlias(alias);
-                  if (topic == null) {
-                     topic = message.variableHeader().topicName();
-                     if (topic == null || topic.length() == 0) {
-                        // using a topic alias with no matching topic in the state; potentially [MQTT-3.3.2-7]
-                        throw new DisconnectException(MQTTReasonCodes.TOPIC_ALIAS_INVALID);
-                     }
-                     session.getState().addClientTopicAlias(alias, topic);
-                  }
                }
-            } else {
-               topic = message.variableHeader().topicName();
+
+               String existingTopicMapping = session.getState().getClientTopicAlias(alias);
+               if (existingTopicMapping == null) {
+                  if (topic == null || topic.isEmpty()) {
+                     // using a topic alias with no matching topic in the state; potentially [MQTT-3.3.2-7]
+                     throw new DisconnectException(MQTTReasonCodes.TOPIC_ALIAS_INVALID);
+                  }
+                  logger.debug("Adding new alias {} for topic {}", alias, topic);
+                  session.getState().putClientTopicAlias(alias, topic);
+               } else if (topic != null && !topic.isEmpty()) {
+                  logger.debug("Modifying existing alias {}. New value: {}; old value: {}", alias, topic, existingTopicMapping);
+                  session.getState().putClientTopicAlias(alias, topic);
+               } else {
+                  logger.debug("Applying topic {} for alias {}", existingTopicMapping, alias);
+                  topic = existingTopicMapping;
+               }
             }
          }
-         String coreAddress = MQTTUtil.convertMqttTopicFilterToCoreAddress(topic, session.getWildcardConfiguration());
-         SimpleString address = SimpleString.toSimpleString(coreAddress, session.getCoreMessageObjectPools().getAddressStringSimpleStringPool());
+         String coreAddress = MQTTUtil.getCoreAddressFromMqttTopic(topic, session.getWildcardConfiguration());
+         SimpleString address = SimpleString.of(coreAddress, session.getCoreMessageObjectPools().getAddressStringSimpleStringPool());
          Message serverMessage = MQTTUtil.createServerMessageFromByteBuf(session, address, message);
          int qos = message.fixedHeader().qosLevel().value();
          if (qos > 0) {
             serverMessage.setDurable(MQTTUtil.DURABLE_MESSAGES);
          }
-         int messageId = message.variableHeader().packetId();
-         if (qos < 2 || !state.getPubRec().contains(messageId)) {
+         int packetId = message.variableHeader().packetId();
+         boolean qos2PublishAlreadyReceived = state.getPubRec().contains(packetId);
+         if (qos < 2 || !qos2PublishAlreadyReceived) {
             if (qos == 2 && !internal)
-               state.getPubRec().add(messageId);
+               state.getPubRec().add(packetId);
 
             Transaction tx = session.getServerSession().newTransaction();
             try {
@@ -246,7 +253,7 @@ public class MQTTPublishManager {
                   throw e;
                }
                if (session.getVersion() == MQTTVersion.MQTT_5) {
-                  sendMessageAck(internal, qos, messageId, MQTTReasonCodes.NOT_AUTHORIZED);
+                  sendMessageAck(internal, qos, packetId, MQTTReasonCodes.NOT_AUTHORIZED);
                   return;
                } else if (session.getVersion() == MQTTVersion.MQTT_3_1_1) {
                   /*
@@ -281,9 +288,11 @@ public class MQTTPublishManager {
                tx.rollback();
                throw t;
             }
+         } else if (qos2PublishAlreadyReceived) {
+            MQTTLogger.LOGGER.ignoringQoS2Publish(state.getClientId(), packetId);
          }
 
-         createMessageAck(messageId, qos, internal);
+         createMessageAck(packetId, qos, internal);
       }
    }
 
@@ -331,8 +340,6 @@ public class MQTTPublishManager {
 
    /**
     * Once we get an acknowledgement for a QoS 1 or 2 message we allow messages to flow
-    *
-    * @param consumerId
     */
    private void releaseFlowControl(Long consumerId) {
       ServerConsumer consumer = session.getServerSession().locateConsumer(consumerId);
@@ -383,7 +390,7 @@ public class MQTTPublishManager {
    }
 
    private boolean publishToClient(int messageId, ICoreMessage message, int deliveryCount, int qos, long consumerId) throws Exception {
-      String address = MQTTUtil.convertCoreAddressToMqttTopicFilter(message.getAddress() == null ? "" : message.getAddress(), session.getWildcardConfiguration());
+      String topic = MQTTUtil.getMqttTopicFromCoreAddress(Objects.requireNonNullElse(message.getAddress(), ""), session.getWildcardConfiguration());
 
       ByteBuf payload;
       switch (message.getType()) {
@@ -404,31 +411,35 @@ public class MQTTPublishManager {
       // [MQTT-3.3.1-2] The DUP flag MUST be set to 0 for all QoS 0 messages.
       boolean redelivery = qos == 0 ? false : (deliveryCount > 1);
 
-      boolean isRetain = message.getBooleanProperty(MQTT_MESSAGE_RETAIN_KEY);
-      MqttProperties mqttProperties = getPublishProperties(message);
+      boolean isRetain = message.containsProperty(MQTT_MESSAGE_RETAIN_INITIAL_DISTRIBUTION_KEY);
+      MqttProperties mqttProperties = null;
 
       if (session.getVersion() == MQTTVersion.MQTT_5) {
-         if (session.getState().getSubscription(message.getAddress()) != null && !session.getState().getSubscription(message.getAddress()).option().isRetainAsPublished()) {
-            isRetain = false;
+         mqttProperties = getPublishProperties(message);
+         if (!isRetain && message.getBooleanProperty(MQTT_MESSAGE_RETAIN_KEY)) {
+            MqttTopicSubscription sub = session.getState().getSubscription(topic);
+            if (sub != null && sub.option().isRetainAsPublished()) {
+               isRetain = true;
+            }
          }
 
          if (session.getState().getClientTopicAliasMaximum() != null) {
-            Integer alias = session.getState().getServerTopicAlias(address);
+            Integer alias = session.getState().getServerTopicAlias(topic);
             if (alias == null) {
-               alias = session.getState().addServerTopicAlias(address);
+               alias = session.getState().addServerTopicAlias(topic);
                if (alias != null) {
                   mqttProperties.add(new MqttProperties.IntegerProperty(TOPIC_ALIAS.value(), alias));
                }
             } else {
                mqttProperties.add(new MqttProperties.IntegerProperty(TOPIC_ALIAS.value(), alias));
-               address = "";
+               topic = "";
             }
          }
       }
 
-      int remainingLength = MQTTUtil.calculateRemainingLength(address, mqttProperties, payload);
+      int remainingLength = MQTTUtil.calculateRemainingLength(topic, mqttProperties, payload);
       MqttFixedHeader header = new MqttFixedHeader(MqttMessageType.PUBLISH, redelivery, MqttQoS.valueOf(qos), isRetain, remainingLength);
-      MqttPublishVariableHeader varHeader = new MqttPublishVariableHeader(address, messageId, mqttProperties);
+      MqttPublishVariableHeader varHeader = new MqttPublishVariableHeader(topic, messageId, mqttProperties);
       MqttPublishMessage publish = new MqttPublishMessage(header, varHeader, payload);
 
       int maxSize = session.getState().getClientMaxPacketSize();
@@ -468,7 +479,7 @@ public class MQTTPublishManager {
          for (SimpleString propertyName : message.getPropertyNames()) {
             if (propertyName.startsWith(MQTT_USER_PROPERTY_KEY_PREFIX_SIMPLE)) {
                SimpleString[] split = propertyName.split('.');
-               int position = Integer.valueOf(split[4].toString());
+               int position = Integer.parseInt(split[4].toString());
                String key = propertyName.subSeq(MQTT_USER_PROPERTY_KEY_PREFIX_SIMPLE.length() + split[4].length() + 1, propertyName.length()).toString();
                orderedProperties[position] = new MqttProperties.StringPair(key, message.getStringProperty(propertyName));
             }
