@@ -1,30 +1,29 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. See the NOTICE file distributed with
+ * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.activemq.artemis.core.protocol.mqtt;
 
+import java.util.ArrayList;
 import java.util.EnumSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
+import java.util.Objects;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 
-import io.netty.handler.codec.mqtt.MqttProperties;
 import io.netty.handler.codec.mqtt.MqttSubscriptionOption;
 import io.netty.handler.codec.mqtt.MqttTopicSubscription;
 import org.apache.activemq.artemis.api.core.ActiveMQQueueExistsException;
@@ -35,28 +34,25 @@ import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.core.server.ActiveMQMessageBundle;
 import org.apache.activemq.artemis.core.server.BindingQueryResult;
-import org.apache.activemq.artemis.core.server.Consumer;
 import org.apache.activemq.artemis.core.server.Queue;
 import org.apache.activemq.artemis.core.server.ServerConsumer;
 import org.apache.activemq.artemis.core.server.impl.AddressInfo;
 import org.apache.activemq.artemis.utils.CompositeAddress;
 
-import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.SUBSCRIPTION_IDENTIFIER;
 import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.DOLLAR;
-import static org.apache.activemq.artemis.core.protocol.mqtt.MQTTUtil.SLASH;
 import static org.apache.activemq.artemis.reader.MessageUtil.CONNECTION_ID_PROPERTY_NAME_STRING;
 
 public class MQTTSubscriptionManager {
 
    private final MQTTSession session;
 
+   private final MQTTStateManager stateManager;
+
    private final ConcurrentMap<Long, Integer> consumerQoSLevels;
 
    private final ConcurrentMap<String, ServerConsumer> consumers;
 
-   /*
-    * We filter out certain messages (e.g. management messages, notifications)
-    */
+   // We filter out certain messages (e.g. management messages, notifications)
    private final SimpleString messageFilter;
 
    /*
@@ -69,8 +65,9 @@ public class MQTTSubscriptionManager {
 
    private final char anyWords;
 
-   public MQTTSubscriptionManager(MQTTSession session) {
+   public MQTTSubscriptionManager(MQTTSession session, MQTTStateManager stateManager) {
       this.session = session;
+      this.stateManager = stateManager;
 
       singleWord = session.getServer().getConfiguration().getWildcardConfiguration().getSingleWord();
       anyWords = session.getServer().getConfiguration().getWildcardConfiguration().getAnyWords();
@@ -87,14 +84,14 @@ public class MQTTSubscriptionManager {
 
       StringBuilder messageFilter = new StringBuilder(baseFilter);
       messageFilter.append(")");
-      this.messageFilter = new SimpleString(messageFilter.toString());
+      this.messageFilter = SimpleString.of(messageFilter.toString());
 
       // [MQTT-4.7.2-1]
       StringBuilder messageFilterNoDollar = new StringBuilder(baseFilter);
       messageFilterNoDollar.append(" OR ");
       messageFilterNoDollar.append("(").append(FilterConstants.ACTIVEMQ_ADDRESS).append(" LIKE '").append(DOLLAR).append("%')");
       messageFilterNoDollar.append(")");
-      this.messageFilterNoDollar = new SimpleString(messageFilterNoDollar.toString());
+      this.messageFilterNoDollar = SimpleString.of(messageFilterNoDollar.toString());
    }
 
    synchronized void start() throws Exception {
@@ -104,40 +101,39 @@ public class MQTTSubscriptionManager {
    }
 
    private void addSubscription(MqttTopicSubscription subscription, Integer subscriptionIdentifier, boolean initialStart) throws Exception {
-      String topicName = CompositeAddress.extractAddressName(subscription.topicName());
-      String sharedSubscriptionName = null;
+      String rawTopicName = CompositeAddress.extractAddressName(subscription.topicFilter());
+      String parsedTopicName = MQTTUtil.decomposeSharedSubscriptionTopicFilter(rawTopicName).getB();
+      boolean isFullyQualified = CompositeAddress.isFullyQualified(subscription.topicFilter());
+
+      Queue q = createQueueForSubscription(rawTopicName, parsedTopicName, isFullyQualified);
 
-      // if using a shared subscription then parse the subscription name and topic
-      if (topicName.startsWith(MQTTUtil.SHARED_SUBSCRIPTION_PREFIX)) {
-         int slashIndex = topicName.indexOf(SLASH) + 1;
-         sharedSubscriptionName = topicName.substring(slashIndex, topicName.indexOf(SLASH, slashIndex));
-         topicName = topicName.substring(topicName.indexOf(SLASH, slashIndex) + 1);
-      }
       int qos = subscription.qualityOfService().value();
-      String coreAddress = MQTTUtil.convertMqttTopicFilterToCoreAddress(topicName, session.getWildcardConfiguration());
 
-      Queue q = createQueueForSubscription(coreAddress, sharedSubscriptionName);
-
-      if (initialStart) {
-         createConsumerForSubscriptionQueue(q, topicName, qos, subscription.option().isNoLocal(), null);
-      } else {
-         MqttTopicSubscription existingSubscription = session.getState().getSubscription(topicName);
-         if (existingSubscription == null) {
-            createConsumerForSubscriptionQueue(q, topicName, qos, subscription.option().isNoLocal(), null);
+      try {
+         if (initialStart) {
+            createConsumerForSubscriptionQueue(q, parsedTopicName, qos, subscription.option().isNoLocal(), null);
          } else {
-            Long existingConsumerId = consumers.get(topicName).getID();
-            consumerQoSLevels.put(existingConsumerId, qos);
-            if (existingSubscription.option().isNoLocal() != subscription.option().isNoLocal()) {
-               createConsumerForSubscriptionQueue(q, topicName, qos, subscription.option().isNoLocal(), existingConsumerId);
+            MqttTopicSubscription existingSubscription = session.getState().getSubscription(parsedTopicName);
+            if (existingSubscription == null) {
+               createConsumerForSubscriptionQueue(q, parsedTopicName, qos, subscription.option().isNoLocal(), null);
+            } else {
+               Long existingConsumerId = consumers.get(parsedTopicName).getID();
+               consumerQoSLevels.put(existingConsumerId, qos);
+               if (existingSubscription.option().isNoLocal() != subscription.option().isNoLocal()) {
+                  createConsumerForSubscriptionQueue(q, parsedTopicName, qos, subscription.option().isNoLocal(), existingConsumerId);
+               }
             }
-         }
 
-         if (subscription.option().retainHandling() == MqttSubscriptionOption.RetainedHandlingPolicy.SEND_AT_SUBSCRIBE ||
-            (subscription.option().retainHandling() == MqttSubscriptionOption.RetainedHandlingPolicy.SEND_AT_SUBSCRIBE_IF_NOT_YET_EXISTS && existingSubscription == null)) {
-            session.getRetainMessageManager().addRetainedMessagesToQueue(q, topicName);
-         }
+            if (subscription.option().retainHandling() == MqttSubscriptionOption.RetainedHandlingPolicy.SEND_AT_SUBSCRIBE || (subscription.option().retainHandling() == MqttSubscriptionOption.RetainedHandlingPolicy.SEND_AT_SUBSCRIBE_IF_NOT_YET_EXISTS && existingSubscription == null)) {
+               session.getRetainMessageManager().addRetainedMessagesToQueue(q, parsedTopicName);
+            }
 
-         session.getState().addSubscription(subscription, session.getWildcardConfiguration(), subscriptionIdentifier);
+            session.getState().addSubscription(subscription, session.getWildcardConfiguration(), subscriptionIdentifier);
+         }
+      } catch (Exception e) {
+         // if anything broke during the creation of the consumer (or otherwise) then ensure the subscription queue
+         q.deleteQueue();
+         throw e;
       }
    }
 
@@ -150,25 +146,20 @@ public class MQTTSubscriptionManager {
       }
    }
 
-   private Queue createQueueForSubscription(String address, String sharedSubscriptionName) throws Exception {
-      // determine the proper queue name
-      SimpleString queue;
-      if (sharedSubscriptionName != null) {
-         queue = SimpleString.toSimpleString(sharedSubscriptionName);
-      } else {
-         queue = getQueueNameForTopic(address);
-      }
+   private Queue createQueueForSubscription(String rawTopicName, String parsedTopicName, boolean isFullyQualified) throws Exception {
+      String coreAddress = MQTTUtil.getCoreAddressFromMqttTopic(parsedTopicName, session.getWildcardConfiguration());
+      String coreQueue = MQTTUtil.getCoreQueueFromMqttTopic(rawTopicName, session.getState().getClientId(), session.getWildcardConfiguration());
 
-      // check to see if a subscription queue already exists.
-      Queue q = session.getServer().locateQueue(queue);
+      // check to see if a subscription queue already exists
+      Queue q = session.getServer().locateQueue(coreQueue);
 
-      // The queue does not exist so we need to create it.
+      // the subscription queue does not exist so we need to create it
       if (q == null) {
-         SimpleString sAddress = SimpleString.toSimpleString(address);
+         SimpleString sAddress = SimpleString.of(coreAddress);
 
-         // Check we can auto create queues.
+         // only check if we can auto create queues if it's FQQN
          BindingQueryResult bindingQueryResult = session.getServerSession().executeBindingQuery(sAddress);
-         if (!bindingQueryResult.isAutoCreateQueues()) {
+         if (isFullyQualified && !bindingQueryResult.isAutoCreateQueues()) {
             throw ActiveMQMessageBundle.BUNDLE.noSuchQueue(sAddress);
          }
 
@@ -176,17 +167,16 @@ public class MQTTSubscriptionManager {
          AddressInfo addressInfo = session.getServerSession().getAddress(sAddress);
          if (addressInfo == null) {
             if (!bindingQueryResult.isAutoCreateAddresses()) {
-               throw ActiveMQMessageBundle.BUNDLE.addressDoesNotExist(SimpleString.toSimpleString(address));
+               throw ActiveMQMessageBundle.BUNDLE.addressDoesNotExist(sAddress);
             }
-            addressInfo = session.getServerSession().createAddress(SimpleString.toSimpleString(address),
-                                                                   RoutingType.MULTICAST, true);
+            addressInfo = session.getServerSession().createAddress(sAddress, RoutingType.MULTICAST, true);
          }
-         return findOrCreateQueue(bindingQueryResult, addressInfo, queue);
+         return findOrCreateQueue(bindingQueryResult, addressInfo, coreQueue);
       }
       return q;
    }
 
-   private Queue findOrCreateQueue(BindingQueryResult bindingQueryResult, AddressInfo addressInfo, SimpleString queue) throws Exception {
+   private Queue findOrCreateQueue(BindingQueryResult bindingQueryResult, AddressInfo addressInfo, String queue) throws Exception {
       /*
        * MQTT 3.1 and 3.1.1 clients using a clean session should have a *non-durable* subscription queue. If the broker
        * restarts the queue should be removed. This is due to [MQTT-3.1.2-6] which states that the session (and any
@@ -194,7 +184,7 @@ public class MQTTSubscriptionManager {
        */
       boolean durable = session.getVersion() == MQTTVersion.MQTT_5 || (session.getVersion() != MQTTVersion.MQTT_5 && !session.isClean());
       if (addressInfo.getRoutingTypes().contains(RoutingType.MULTICAST)) {
-         return session.getServerSession().createQueue(new QueueConfiguration(queue).setAddress(addressInfo.getName()).setFilterString(getMessageFilter(addressInfo.getName())).setDurable(durable));
+         return session.getServerSession().createQueue(QueueConfiguration.of(queue).setAddress(addressInfo.getName()).setFilterString(getMessageFilter(addressInfo.getName())).setDurable(durable));
       }
 
       if (addressInfo.getRoutingTypes().contains(RoutingType.ANYCAST)) {
@@ -210,7 +200,7 @@ public class MQTTSubscriptionManager {
             return session.getServer().locateQueue(name);
          } else {
             try {
-               return session.getServerSession().createQueue(new QueueConfiguration(addressInfo.getName()).setRoutingType(RoutingType.ANYCAST).setFilterString(getMessageFilter(addressInfo.getName())).setDurable(durable));
+               return session.getServerSession().createQueue(QueueConfiguration.of(addressInfo.getName()).setRoutingType(RoutingType.ANYCAST).setFilterString(getMessageFilter(addressInfo.getName())).setDurable(durable));
             } catch (ActiveMQQueueExistsException e) {
                return session.getServer().locateQueue(addressInfo.getName());
             }
@@ -232,13 +222,13 @@ public class MQTTSubscriptionManager {
       }
    }
 
-   private void createConsumerForSubscriptionQueue(Queue queue, String topic, int qos, boolean noLocal, Long existingConsumerId) throws Exception {
-      long cid = existingConsumerId != null ? existingConsumerId : session.getServer().getStorageManager().generateID();
+   private void createConsumerForSubscriptionQueue(Queue queue, String topicFilter, int qos, boolean noLocal, Long existingConsumerId) throws Exception {
+      long cid = Objects.requireNonNullElseGet(existingConsumerId, () -> session.getServer().getStorageManager().generateID());
 
       // for noLocal support we use the MQTT *client id* rather than the connection ID, but we still use the existing property name
-      ServerConsumer consumer = session.getServerSession().createConsumer(cid, queue.getName(), noLocal ? SimpleString.toSimpleString(CONNECTION_ID_PROPERTY_NAME_STRING + " <> '" + session.getState().getClientId() + "'") : null, false, false, -1);
+      ServerConsumer consumer = session.getServerSession().createConsumer(cid, queue.getName(), noLocal ? SimpleString.of(CONNECTION_ID_PROPERTY_NAME_STRING + " <> '" + session.getState().getClientId() + "'") : null, false, false, -1);
 
-      ServerConsumer existingConsumer = consumers.put(topic, consumer);
+      ServerConsumer existingConsumer = consumers.put(topicFilter, consumer);
       if (existingConsumer != null) {
          existingConsumer.setStarted(false);
          existingConsumer.close(false);
@@ -249,89 +239,70 @@ public class MQTTSubscriptionManager {
       consumerQoSLevels.put(cid, qos);
    }
 
-   short[] removeSubscriptions(List<String> topics) throws Exception {
-      short[] reasonCodes;
+   short[] removeSubscriptions(List<String> topics, boolean enforceSecurity) throws Exception {
+      if (topics == null || topics.size() == 0) {
+         return new short[0];
+      }
 
-      synchronized (session.getState()) {
+      short[] reasonCodes;
+      MQTTSessionState state = session.getState();
+
+      synchronized (state) {
          reasonCodes = new short[topics.size()];
          for (int i = 0; i < topics.size(); i++) {
-            reasonCodes[i] = removeSubscription(topics.get(i));
+            if (state.getSubscription(topics.get(i)) == null) {
+               reasonCodes[i] = MQTTReasonCodes.NO_SUBSCRIPTION_EXISTED;
+               continue;
+            }
+
+            short reasonCode = MQTTReasonCodes.SUCCESS;
+
+            try {
+               state.removeSubscription(topics.get(i));
+               ServerConsumer removed = consumers.remove(MQTTUtil.decomposeSharedSubscriptionTopicFilter(topics.get(i)).getB());
+               if (removed != null) {
+                  removed.close(false);
+                  consumerQoSLevels.remove(removed.getID());
+               }
+
+               SimpleString internalQueueName = SimpleString.of(MQTTUtil.getCoreQueueFromMqttTopic(topics.get(i), state.getClientId(), session.getServer().getConfiguration().getWildcardConfiguration()));
+               Queue queue = session.getServer().locateQueue(internalQueueName);
+               if (queue != null) {
+                  if (queue.isConfigurationManaged()) {
+                     queue.deleteAllReferences();
+                  } else if (!MQTTUtil.isSharedSubscription(topics.get(i)) || (MQTTUtil.isSharedSubscription(topics.get(i)) && queue.getConsumerCount() == 0)) {
+                     session.getServerSession().deleteQueue(internalQueueName, enforceSecurity);
+                  }
+               }
+            } catch (Exception e) {
+               MQTTLogger.LOGGER.errorRemovingSubscription(e);
+               reasonCode = MQTTReasonCodes.UNSPECIFIED_ERROR;
+            }
+
+            reasonCodes[i] =  reasonCode;
+         }
+
+         // deal with durable state after *all* requested subscriptions have been removed in memory
+         if (state.getSubscriptions().size() > 0) {
+            // if there are some subscriptions left then update the state
+            stateManager.storeDurableSubscriptionState(state);
+         } else {
+            // if there are no subscriptions left then remove the state entirely
+            stateManager.removeDurableSubscriptionState(state.getClientId());
          }
       }
 
       return reasonCodes;
    }
 
-   private short removeSubscription(String address) {
-      if (session.getState().getSubscription(address) == null) {
-         return MQTTReasonCodes.NO_SUBSCRIPTION_EXISTED;
-      }
-
-      short reasonCode = MQTTReasonCodes.SUCCESS;
-
-      try {
-         String internalAddress = MQTTUtil.convertMqttTopicFilterToCoreAddress(address, session.getWildcardConfiguration());
-         SimpleString internalQueueName = getQueueNameForTopic(internalAddress);
-         session.getState().removeSubscription(address);
-
-         Queue queue = session.getServer().locateQueue(internalQueueName);
-         SimpleString sAddress = SimpleString.toSimpleString(internalAddress);
-         AddressInfo addressInfo = session.getServerSession().getAddress(sAddress);
-         if (addressInfo != null && addressInfo.getRoutingTypes().contains(RoutingType.ANYCAST)) {
-            ServerConsumer consumer = consumers.get(address);
-            consumers.remove(address);
-            if (consumer != null) {
-               consumer.close(false);
-               consumerQoSLevels.remove(consumer.getID());
-            }
-         } else {
-            consumers.remove(address);
-            Set<Consumer> queueConsumers;
-            if (queue != null && (queueConsumers = (Set<Consumer>) queue.getConsumers()) != null) {
-               for (Consumer consumer : queueConsumers) {
-                  if (consumer instanceof ServerConsumer) {
-                     ((ServerConsumer) consumer).close(false);
-                     consumerQoSLevels.remove(((ServerConsumer) consumer).getID());
-                  }
-               }
-            }
-         }
-
-         if (queue != null) {
-            assert session.getServerSession().executeQueueQuery(internalQueueName).isExists();
-
-            if (queue.isConfigurationManaged()) {
-               queue.deleteAllReferences();
-            } else {
-               session.getServerSession().deleteQueue(internalQueueName);
-            }
-         }
-      } catch (Exception e) {
-         MQTTLogger.LOGGER.errorRemovingSubscription(e);
-         reasonCode = MQTTReasonCodes.UNSPECIFIED_ERROR;
-      }
-
-      return reasonCode;
-   }
-
-   private SimpleString getQueueNameForTopic(String topic) {
-      return new SimpleString(session.getState().getClientId() + "." + topic);
-   }
-
    /**
-    * As per MQTT Spec.  Subscribes this client to a number of MQTT topics.
+    * As per MQTT Spec. Subscribes this client to a number of MQTT topics.
     *
-    * @param subscriptions
-    * @return An array of integers representing the list of accepted QoS for each topic.
-    * @throws Exception
+    * @return An array of integers representing the list of accepted QoS for each topic
     */
-   int[] addSubscriptions(List<MqttTopicSubscription> subscriptions, MqttProperties properties) throws Exception {
-      synchronized (session.getState()) {
-         Integer subscriptionIdentifier = null;
-         if (properties.getProperty(SUBSCRIPTION_IDENTIFIER.value()) != null) {
-            subscriptionIdentifier = (Integer) properties.getProperty(SUBSCRIPTION_IDENTIFIER.value()).value();
-         }
-
+   int[] addSubscriptions(List<MqttTopicSubscription> subscriptions, Integer subscriptionIdentifier) throws Exception {
+      MQTTSessionState state = session.getState();
+      synchronized (state) {
          int[] qos = new int[subscriptions.size()];
 
          for (int i = 0; i < subscriptions.size(); i++) {
@@ -339,7 +310,7 @@ public class MQTTSubscriptionManager {
                addSubscription(subscriptions.get(i), subscriptionIdentifier, false);
                qos[i] = subscriptions.get(i).qualityOfService().value();
             } catch (ActiveMQSecurityException e) {
-               // user is not authorized to create subsription
+               // user is not authorized to create subscription
                if (session.getVersion() == MQTTVersion.MQTT_5) {
                   qos[i] = MQTTReasonCodes.NOT_AUTHORIZED;
                } else if (session.getVersion() == MQTTVersion.MQTT_3_1_1) {
@@ -365,6 +336,10 @@ public class MQTTSubscriptionManager {
                }
             }
          }
+
+         // store state after *all* requested subscriptions have been created in memory
+         stateManager.storeDurableSubscriptionState(state);
+
          return qos;
       }
    }
@@ -373,9 +348,11 @@ public class MQTTSubscriptionManager {
       return consumerQoSLevels;
    }
 
-   void clean() {
+   void clean(boolean enforceSecurity) throws Exception {
+      List<String> topics = new ArrayList<>();
       for (MqttTopicSubscription mqttTopicSubscription : session.getState().getSubscriptions()) {
-         removeSubscription(mqttTopicSubscription.topicName());
+         topics.add(mqttTopicSubscription.topicFilter());
       }
+      removeSubscriptions(topics, enforceSecurity);
    }
 }
