@@ -16,6 +16,7 @@
  */
 package org.apache.activemq.artemis.core.remoting.server.impl;
 
+import java.lang.invoke.MethodHandles;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Collections;
@@ -26,8 +27,8 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.ServiceLoader;
 import java.util.Set;
-import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Executor;
@@ -80,7 +81,6 @@ import org.apache.activemq.artemis.utils.ConfigurationHelper;
 import org.apache.activemq.artemis.utils.ReusableLatch;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import java.lang.invoke.MethodHandles;
 
 public class RemotingServiceImpl implements RemotingService, ServerConnectionLifeCycleListener {
 
@@ -202,14 +202,7 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       // This needs to be a different thread pool to the main thread pool especially for OIO where we may need
       // to support many hundreds of connections, but the main thread pool must be kept small for better performance
 
-      ThreadFactory tFactory = AccessController.doPrivileged(new PrivilegedAction<ThreadFactory>() {
-         @Override
-         public ThreadFactory run() {
-            return new ActiveMQThreadFactory("ActiveMQ-remoting-threads-" + server.toString() +
-                                                "-" +
-                                                System.identityHashCode(this), false, Thread.currentThread().getContextClassLoader());
-         }
-      });
+      ThreadFactory tFactory = AccessController.doPrivileged((PrivilegedAction<ThreadFactory>) () -> new ActiveMQThreadFactory("ActiveMQ-remoting-threads-" + server.toString() + "-" + System.identityHashCode(this), false, Thread.currentThread().getContextClassLoader()));
 
       threadPool = Executors.newCachedThreadPool(tFactory);
 
@@ -217,7 +210,7 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
          createAcceptor(info);
       }
 
-      /**
+      /*
        * Don't start the acceptors here.  Only start the acceptors at the every end of the start-up process to avoid
        * race conditions. See {@link #startAcceptors()}.
        */
@@ -285,7 +278,7 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
             managementService.registerAcceptor(acceptor, info);
          }
       } catch (Exception e) {
-         ActiveMQServerLogger.LOGGER.errorCreatingAcceptor(info.getFactoryClassName(), e);
+         ActiveMQServerLogger.LOGGER.errorCreatingAcceptor(info.getName(), e);
       }
 
       return acceptor;
@@ -310,7 +303,7 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       if (isStarted()) {
          for (Acceptor a : acceptors.values()) {
             try {
-               if (a instanceof NettyAcceptor && !((NettyAcceptor)a).isAutoStart()) {
+               if (a instanceof NettyAcceptor acceptor && !acceptor.isAutoStart()) {
                   continue;
                }
                a.start();
@@ -358,7 +351,7 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       if (!started)
          return;
       failureCheckAndFlushThread.close(false);
-      HashMap<Object, ConnectionEntry> connectionEntries = new HashMap<>(connections);
+      Map<Object, ConnectionEntry> connectionEntries = new HashMap<>(connections);
 
       // Now we ensure that no connections will process any more packets after this method is
       // complete then send a disconnect packet
@@ -379,17 +372,22 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
    }
 
    @Override
-   public void stop(final boolean criticalError) throws Exception {
-      if (!started) {
-         return;
-      }
-      SSLContextFactoryProvider.getSSLContextFactory().clearSSLContexts();
-      OpenSSLContextFactory openSSLContextFactory = OpenSSLContextFactoryProvider.getOpenSSLContextFactory();
-      if (openSSLContextFactory != null) {
-         openSSLContextFactory.clearSslContexts();
-      }
+   public void notifyStop() {
 
-      failureCheckAndFlushThread.close(criticalError);
+      // We need to stop them accepting first so no new connections are accepted after we send the disconnect message
+      for (Acceptor acceptor : acceptors.values()) {
+         logger.debug("send stop notifications on acceptor {}", acceptor);
+
+         try {
+            acceptor.notifyStop();
+         } catch (Throwable t) {
+            ActiveMQServerLogger.LOGGER.errorStoppingAcceptor(acceptor.getName());
+         }
+      }
+   }
+
+   @Override
+   public void prepareStop(boolean criticalError, Set<RemotingConnection> ignoreList) throws Exception {
 
       // We need to stop them accepting first so no new connections are accepted after we send the disconnect message
       for (Acceptor acceptor : acceptors.values()) {
@@ -403,20 +401,44 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
 
       }
 
-      logger.debug("Sending disconnect on live connections");
+      logger.debug("Sending disconnect on client connections");
 
-      HashSet<ConnectionEntry> connectionEntries = new HashSet<>(connections.values());
+      Set<ConnectionEntry> connectionEntries = new HashSet<>(connections.values());
 
       // Now we ensure that no connections will process any more packets after this method is complete
       // then send a disconnect packet
       for (ConnectionEntry entry : connectionEntries) {
          RemotingConnection conn = entry.connection;
 
-         logger.trace("Sending connection.disconnection packet to {}", conn);
-
-         conn.disconnect(criticalError);
+         if (ignoreList.contains(conn)) {
+            logger.debug("ignoring connection {} during the close", conn);
+         } else {
+            logger.debug("Sending disconnect on connection {} from server {}", conn.getID(), server);
+            conn.disconnect(criticalError);
+         }
       }
 
+   }
+
+   @Override
+   public void stop(final boolean criticalError) throws Exception {
+      if (!started) {
+         return;
+      }
+      SSLContextFactoryProvider.getSSLContextFactory().clearSSLContexts();
+      OpenSSLContextFactory openSSLContextFactory = OpenSSLContextFactoryProvider.getOpenSSLContextFactory();
+      if (openSSLContextFactory != null) {
+         openSSLContextFactory.clearSslContexts();
+      }
+
+      failureCheckAndFlushThread.close(criticalError);
+
+      // ActiveMQServerImpl already calls prepareStop
+      // however we call this again here for two reasons:
+      // I - ActiveMQServer might have ignored the one connection for Replication
+      // II - this method could be called in other places for Embedding or testing and the semantic must be kept the same
+      prepareStop(criticalError, Collections.emptySet());
+
       CountDownLatch acceptorCountDownLatch = new CountDownLatch(acceptors.size());
       for (Acceptor acceptor : acceptors.values()) {
          try {
@@ -460,7 +482,8 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       return started;
    }
 
-   private RemotingConnection getConnection(final Object remotingConnectionID) {
+   @Override
+   public RemotingConnection getConnection(final Object remotingConnectionID) {
       ConnectionEntry entry = connections.get(remotingConnectionID);
 
       if (entry != null) {
@@ -546,6 +569,13 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       }
    }
 
+   @Override
+   public void updateProtocolServices(List<ActiveMQComponent> protocolServices) throws Exception {
+      for (ProtocolManagerFactory protocolManagerFactory : protocolMap.values()) {
+         protocolManagerFactory.updateProtocolServices(this.server, protocolServices);
+      }
+   }
+
    // ServerConnectionLifeCycleListener implementation -----------------------------------
 
    private ProtocolManagerFactory getProtocolManager(String protocol) {
@@ -582,20 +612,24 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
    public void addConnectionEntry(Connection connection, ConnectionEntry entry) {
       connections.put(connection.getID(), entry);
       if (AuditLogger.isResourceLoggingEnabled()) {
-         AuditLogger.createdConnection(connection.getProtocolConnection().getProtocolName(), connection.getID(), connection.getRemoteAddress());
+         AuditLogger.createdConnection(connection.getProtocolConnection() == null ? null : connection.getProtocolConnection().getProtocolName(), connection.getID(), connection.getRemoteAddress());
       }
       if (logger.isDebugEnabled()) {
-         logger.debug("Adding connection {}, we now have {}", connection.getID(), connections.size());
+         logger.debug("Adding connection {}, we now have {} on server {}", connection.getID(), connections.size(), server);
       }
    }
 
    @Override
-   public void connectionDestroyed(final Object connectionID) {
+   public void connectionDestroyed(final Object connectionID, boolean failed) {
       if (logger.isTraceEnabled()) {
          logger.trace("Connection removed {} from server {}", connectionID, this.server, new Exception("trace"));
       }
 
-      issueFailure(connectionID, new ActiveMQRemoteDisconnectException());
+      if (failed) {
+         issueFailure(connectionID, new ActiveMQRemoteDisconnectException());
+      } else {
+         issueClose(connectionID);
+      }
    }
 
    private void issueFailure(Object connectionID, ActiveMQException e) {
@@ -618,6 +652,26 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       }
    }
 
+   private void issueClose(Object connectionID) {
+      ConnectionEntry conn = connections.get(connectionID);
+
+      if (conn != null && !conn.connection.isSupportReconnect()) {
+         RemotingConnection removedConnection = removeConnection(connectionID);
+         if (removedConnection != null) {
+            try {
+               if (server.hasBrokerConnectionPlugins()) {
+                  server.callBrokerConnectionPlugins(plugin -> plugin.afterDestroyConnection(removedConnection));
+               }
+            } catch (ActiveMQException t) {
+               logger.warn("Error executing afterDestroyConnection plugin method: {}", t.getMessage(), t);
+            }
+         }
+         conn.connection.close();
+      }
+   }
+
+
+
    @Override
    public void connectionException(final Object connectionID, final ActiveMQException me) {
       issueFailure(connectionID, me);
@@ -737,19 +791,19 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
       public void run() {
          while (!closed) {
             try {
-               long now = System.currentTimeMillis();
-
                Set<Pair<Object, Long>> toRemove = new HashSet<>();
-
                for (ConnectionEntry entry : connections.values()) {
                   final RemotingConnection conn = entry.connection;
+                  final long lastCheck = entry.lastCheck;
+                  final long ttl = entry.ttl;
+                  final long now = System.currentTimeMillis();
 
                   boolean flush = true;
 
-                  if (entry.ttl != -1) {
+                  if (ttl != -1) {
                      if (!conn.checkDataReceived()) {
-                        if (now >= entry.lastCheck + entry.ttl) {
-                           toRemove.add(new Pair<>(conn.getID(), entry.ttl));
+                        if (now >= lastCheck + ttl) {
+                           toRemove.add(new Pair<>(conn.getID(), ttl));
 
                            flush = false;
                         }
@@ -759,19 +813,16 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
                   }
 
                   if (flush) {
-                     flushExecutor.execute(new Runnable() {
-                        @Override
-                        public void run() {
-                           try {
-                              // this is using a different thread
-                              // as if anything wrong happens on flush
-                              // failure detection could be affected
-                              conn.scheduledFlush();
-                           } catch (Throwable e) {
-                              ActiveMQServerLogger.LOGGER.failedToFlushOutstandingDataFromTheConnection(e);
-                           }
-
+                     flushExecutor.execute(() -> {
+                        try {
+                           // this is using a different thread
+                           // as if anything wrong happens on flush
+                           // failure detection could be affected
+                           conn.scheduledFlush();
+                        } catch (Throwable e) {
+                           ActiveMQServerLogger.LOGGER.failedToFlushOutstandingDataFromTheConnection(e);
                         }
+
                      });
                   }
                }
@@ -782,12 +833,7 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
                      // In certain cases (replicationManager for instance) calling fail could take some time
                      // We can't pause the FailureCheckAndFlushThread as that would lead other clients to fail for
                      // missing pings
-                     flushExecutor.execute(new Runnable() {
-                        @Override
-                        public void run() {
-                           conn.fail(ActiveMQMessageBundle.BUNDLE.clientExited(conn.getRemoteAddress(), pair.getB()));
-                        }
-                     });
+                     flushExecutor.execute(() -> conn.fail(ActiveMQMessageBundle.BUNDLE.clientExited(conn.getRemoteAddress(), pair.getB())));
                      removeConnection(pair.getA());
                   }
                }
@@ -809,10 +855,6 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
 
    /**
     * Locates protocols from the internal default map and moves them into the input protocol map.
-    *
-    * @param protocolList
-    * @param transportConfig
-    * @param protocolMap
     */
    private void locateProtocols(String protocolList,
                                 Object transportConfig,
@@ -832,8 +874,6 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
 
    /**
     * Finds protocol support from a given classloader.
-    *
-    * @param loader
     */
    private void resolveProtocols(ClassLoader loader) {
       ServiceLoader<ProtocolManagerFactory> serviceLoader = ServiceLoader.load(ProtocolManagerFactory.class, loader);
@@ -842,8 +882,6 @@ public class RemotingServiceImpl implements RemotingService, ServerConnectionLif
 
    /**
     * Loads the protocols found into a map.
-    *
-    * @param protocolManagerFactoryCollection
     */
    private void loadProtocolManagerFactories(Iterable<ProtocolManagerFactory> protocolManagerFactoryCollection) {
       for (ProtocolManagerFactory next : protocolManagerFactoryCollection) {
