@@ -19,15 +19,18 @@ package org.apache.activemq.artemis.core.server.cluster.impl;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ScheduledExecutorService;
 
 import org.apache.activemq.artemis.api.config.ActiveMQDefaultConfiguration;
+import org.apache.activemq.artemis.api.core.ActiveMQAddressDoesNotExistException;
 import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.ActiveMQExceptionType;
 import org.apache.activemq.artemis.api.core.Message;
 import org.apache.activemq.artemis.api.core.QueueConfiguration;
+import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.TransportConfiguration;
 import org.apache.activemq.artemis.api.core.client.ClientConsumer;
@@ -39,7 +42,6 @@ import org.apache.activemq.artemis.api.core.management.ManagementHelper;
 import org.apache.activemq.artemis.api.core.management.ResourceNames;
 import org.apache.activemq.artemis.core.client.impl.ClientSessionFactoryInternal;
 import org.apache.activemq.artemis.core.client.impl.ServerLocatorInternal;
-import org.apache.activemq.artemis.core.client.impl.TopologyMemberImpl;
 import org.apache.activemq.artemis.core.config.BridgeConfiguration;
 import org.apache.activemq.artemis.core.config.TransformerConfiguration;
 import org.apache.activemq.artemis.core.filter.Filter;
@@ -56,7 +58,9 @@ import org.apache.activemq.artemis.core.server.cluster.MessageFlowRecord;
 import org.apache.activemq.artemis.utils.CompositeAddress;
 import org.apache.activemq.artemis.utils.UUID;
 import org.apache.activemq.artemis.utils.UUIDGenerator;
-import org.jboss.logging.Logger;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import java.lang.invoke.MethodHandles;
 
 /**
  * A bridge with extra functionality only available when the server is clustered.
@@ -64,7 +68,7 @@ import org.jboss.logging.Logger;
  * Such as such adding extra properties and setting up notifications between the nodes.
  */
 public class ClusterConnectionBridge extends BridgeImpl {
-   private static final Logger logger = Logger.getLogger(ClusterConnectionBridge.class);
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
    private final ClusterConnection clusterConnection;
 
@@ -89,8 +93,6 @@ public class ClusterConnectionBridge extends BridgeImpl {
 
    private final String storeAndForwardPrefix;
 
-   private TopologyMemberImpl member;
-
    public ClusterConnectionBridge(final ClusterConnection clusterConnection,
                                   final ClusterManager clusterManager,
                                   final ServerLocatorInternal targetLocator,
@@ -119,22 +121,24 @@ public class ClusterConnectionBridge extends BridgeImpl {
                                   final MessageFlowRecord flowRecord,
                                   final TransportConfiguration connector,
                                   final String storeAndForwardPrefix,
-                                  final StorageManager storageManager) throws ActiveMQException {
+                                  final StorageManager storageManager,
+                                  final String clientId) throws ActiveMQException {
       super(targetLocator, new BridgeConfiguration()
-         .setName(name == null ? null : name.toString())
+         .setName(Objects.toString(name, null))
          .setInitialConnectAttempts(initialConnectAttempts)
          .setReconnectAttempts(reconnectAttempts)
          .setReconnectAttemptsOnSameNode(0) // reconnectAttemptsOnSameNode means nothing on the clustering bridge since we always try the same
          .setRetryInterval(retryInterval)
          .setRetryIntervalMultiplier(retryMultiplier)
          .setMaxRetryInterval(maxRetryInterval)
-         .setFilterString(filterString == null ? null : filterString.toString())
-         .setForwardingAddress(forwardingAddress == null ? null : forwardingAddress.toString())
+         .setFilterString(Objects.toString(filterString, null))
+         .setForwardingAddress(Objects.toString(forwardingAddress, null))
          .setUseDuplicateDetection(useDuplicateDetection)
          .setUser(user)
          .setPassword(password)
          .setTransformerConfiguration(transformer)
-         .setRoutingType(ComponentConfigurationRoutingType.valueOf(ActiveMQDefaultConfiguration.getDefaultBridgeRoutingType())), nodeUUID, queue, executor, scheduledExecutor, server);
+         .setRoutingType(ComponentConfigurationRoutingType.valueOf(ActiveMQDefaultConfiguration.getDefaultBridgeRoutingType()))
+         .setClientId(clientId), nodeUUID, queue, executor, scheduledExecutor, server);
 
       this.discoveryLocator = discoveryLocator;
 
@@ -151,7 +155,7 @@ public class ClusterConnectionBridge extends BridgeImpl {
       this.flowRecord = flowRecord;
 
       if (logger.isTraceEnabled()) {
-         logger.trace("Setting up bridge between " + clusterConnection.getConnector() + " and " + targetLocator, new Exception("trace"));
+         logger.trace("Setting up bridge between {} and {}", clusterConnection.getConnector(), targetLocator, new Exception("trace"));
       }
 
       this.storeAndForwardPrefix = storeAndForwardPrefix;
@@ -189,9 +193,7 @@ public class ClusterConnectionBridge extends BridgeImpl {
       // Note we must copy since same message may get routed to other nodes which require different headers
       Message messageCopy = message.copy();
 
-      if (logger.isTraceEnabled()) {
-         logger.trace("Clustered bridge  copied message " + message + " as " + messageCopy + " before delivery");
-      }
+      logger.trace("Clustered bridge  copied message {} as {} before delivery", message, messageCopy);
 
       // TODO - we can optimise this
 
@@ -221,21 +223,15 @@ public class ClusterConnectionBridge extends BridgeImpl {
    private void setupNotificationConsumer() throws Exception {
       if (flowRecord != null) {
          if (logger.isDebugEnabled()) {
-            logger.debug("Setting up notificationConsumer between " + this.clusterConnection.getConnector() +
-                            " and " +
-                            flowRecord.getBridge().getForwardingConnection() +
-                            " clusterConnection = " +
-                            this.clusterConnection.getName() +
-                            " on server " +
-                            clusterConnection.getServer());
+            logger.debug("Setting up notificationConsumer between {} and {} clusterConnection = {} on server {}",
+                          this.clusterConnection.getConnector(), flowRecord.getBridge().getForwardingConnection(),
+                          this.clusterConnection.getName(), clusterConnection.getServer());
          }
          flowRecord.reset();
 
          if (notifConsumer != null) {
             try {
-               logger.debug("Closing notification Consumer for reopening " + notifConsumer +
-                               " on bridge " +
-                               this.getName());
+               logger.debug("Closing notification Consumer for reopening {} on bridge {}", notifConsumer, this.getName());
                notifConsumer.close();
 
                notifConsumer = null;
@@ -249,9 +245,9 @@ public class ClusterConnectionBridge extends BridgeImpl {
             "." +
             clusterConnection.getServer().toString().replaceAll(CompositeAddress.SEPARATOR, "_");
 
-         SimpleString notifQueueName = new SimpleString(qName);
+         SimpleString notifQueueName = SimpleString.of(qName);
 
-         SimpleString filter = new SimpleString("(" + ManagementHelper.HDR_BINDING_TYPE + " <> " + BindingType.DIVERT.toInt() +
+         SimpleString filter = SimpleString.of("(" + ManagementHelper.HDR_BINDING_TYPE + " <> " + BindingType.DIVERT.toInt() +
                                                    " OR "
                                                    + ManagementHelper.HDR_BINDING_TYPE + " IS NULL)" +
                                                    " AND " +
@@ -263,6 +259,8 @@ public class ClusterConnectionBridge extends BridgeImpl {
                                                    "', '" +
                                                    CoreNotificationType.BINDING_REMOVED +
                                                    "', '" +
+                                                   CoreNotificationType.BINDING_UPDATED +
+                                                   "', '" +
                                                    CoreNotificationType.CONSUMER_CREATED +
                                                    "', '" +
                                                    CoreNotificationType.CONSUMER_CLOSED +
@@ -283,7 +281,12 @@ public class ClusterConnectionBridge extends BridgeImpl {
                                                    createPermissiveManagementNotificationToFilter() +
                                                    ")");
 
-         sessionConsumer.createQueue(new QueueConfiguration(notifQueueName).setAddress(managementNotificationAddress).setFilterString(filter).setDurable(false).setTemporary(true));
+         sessionConsumer.createQueue(QueueConfiguration.of(notifQueueName)
+                                        .setAddress(managementNotificationAddress)
+                                        .setFilterString(filter)
+                                        .setDurable(false)
+                                        .setTemporary(true)
+                                        .setRoutingType(RoutingType.MULTICAST));
 
          notifConsumer = sessionConsumer.createConsumer(notifQueueName);
 
@@ -293,26 +296,21 @@ public class ClusterConnectionBridge extends BridgeImpl {
 
          ClientMessage message = sessionConsumer.createMessage(false);
          if (logger.isTraceEnabled()) {
-            logger.trace("Requesting sendQueueInfoToQueue through " + this, new Exception("trace"));
+            logger.trace("Requesting sendQueueInfoToQueue through {}", this, new Exception("trace"));
          }
          ManagementHelper.putOperationInvocation(message, ResourceNames.BROKER, "sendQueueInfoToQueue", notifQueueName.toString(), flowRecord.getAddress());
 
          try (ClientProducer prod = sessionConsumer.createProducer(managementAddress)) {
-            if (logger.isDebugEnabled()) {
-               logger.debug("Cluster connection bridge on " + clusterConnection + " requesting information on queues");
-            }
+            logger.debug("Cluster connection bridge on {} requesting information on queues", clusterConnection);
 
             prod.send(message);
          }
       }
    }
 
-
    /**
     * Takes in a string of an address filter or comma separated list and generates an appropriate JMS selector for
     * filtering queues.
-    *
-    * @param address
     */
    public static String createSelectorFromAddress(String address) {
       StringBuilder stringBuilder = new StringBuilder();
@@ -346,28 +344,28 @@ public class ClusterConnectionBridge extends BridgeImpl {
 
       // Build the address matching part of the selector
       StringBuilder builder = new StringBuilder("(");
-      if (includes.size() > 0) {
-         if (excludes.size() > 0)
+      if (!includes.isEmpty()) {
+         if (!excludes.isEmpty())
             builder.append("(");
          for (int i = 0; i < includes.size(); i++) {
             builder.append("(" + ManagementHelper.HDR_ADDRESS + " LIKE '" + includes.get(i) + "%')");
             if (i < includes.size() - 1)
                builder.append(" OR ");
          }
-         if (excludes.size() > 0)
+         if (!excludes.isEmpty())
             builder.append(")");
       }
 
       // Build the address exclusion part of the selector
-      if (excludes.size() > 0) {
-         if (includes.size() > 0)
+      if (!excludes.isEmpty()) {
+         if (!includes.isEmpty())
             builder.append(" AND (");
          for (int i = 0; i < excludes.size(); i++) {
             builder.append("(" + ManagementHelper.HDR_ADDRESS + " NOT LIKE '" + excludes.get(i) + "%')");
             if (i < excludes.size() - 1)
                builder.append(" AND ");
          }
-         if (includes.size() > 0)
+         if (!includes.isEmpty())
             builder.append(")");
       }
       builder.append(")");
@@ -388,9 +386,8 @@ public class ClusterConnectionBridge extends BridgeImpl {
    }
 
    /**
-    * Create a filter rule,in addition to SESSION_CREATED notifications, all other notifications using managementNotificationAddress
-    * as the routing address will be filtered.
-    * @return
+    * Create a filter rule,in addition to SESSION_CREATED notifications, all other notifications using
+    * managementNotificationAddress as the routing address will be filtered.
     */
    private String createPermissiveManagementNotificationToFilter() {
       StringBuilder filterBuilder = new StringBuilder(ManagementHelper.HDR_NOTIFICATION_TYPE).append(" = '")
@@ -427,21 +424,25 @@ public class ClusterConnectionBridge extends BridgeImpl {
 
    @Override
    protected void fail(final boolean permanently, final boolean scaleDown) {
-      logger.debug("Cluster Bridge " + this.getName() + " failed, permanently=" + permanently);
+      logger.debug("Cluster Bridge {} failed, permanently={}", this.getName(), permanently);
       super.fail(permanently, scaleDown);
 
       if (permanently) {
-         logger.debug("cluster node for bridge " + this.getName() + " is permanently down");
+         logger.debug("cluster node for bridge {} is permanently down", this.getName());
          clusterConnection.removeRecord(targetNodeID);
 
          if (scaleDown) {
-            try {
-               queue.deleteQueue(true);
-               queue.removeAddress();
-
-            } catch (Exception e) {
-               logger.warn(e.getMessage(), e);
-            }
+            executor.execute(() -> {
+               logger.debug("Scaling down queue {}", queue);
+               try {
+                  queue.deleteQueue(true);
+                  queue.removeAddress();
+               } catch (ActiveMQAddressDoesNotExistException e) {
+                  logger.debug("ActiveMQAddressDoesNotExistException during scale down for queue {}", queue);
+               } catch (Exception e) {
+                  logger.warn(e.getMessage(), e);
+               }
+            });
          }
       } else {
          clusterConnection.disconnectRecord(targetNodeID);
