@@ -34,7 +34,9 @@ import org.apache.activemq.artemis.api.core.Message;
 import org.apache.activemq.artemis.api.core.Pair;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.core.io.IOCallback;
+import org.apache.activemq.artemis.core.io.OperationConsistencyLevel;
 import org.apache.activemq.artemis.core.io.SequentialFile;
+import org.apache.activemq.artemis.core.journal.IOCompletion;
 import org.apache.activemq.artemis.core.journal.Journal;
 import org.apache.activemq.artemis.core.journal.JournalLoadInformation;
 import org.apache.activemq.artemis.core.journal.RecordInfo;
@@ -47,6 +49,7 @@ import org.apache.activemq.artemis.core.persistence.AddressBindingInfo;
 import org.apache.activemq.artemis.core.persistence.AddressQueueStatus;
 import org.apache.activemq.artemis.core.persistence.GroupingInfo;
 import org.apache.activemq.artemis.core.persistence.OperationContext;
+import org.apache.activemq.artemis.core.persistence.Persister;
 import org.apache.activemq.artemis.core.persistence.QueueBindingInfo;
 import org.apache.activemq.artemis.core.persistence.StorageManager;
 import org.apache.activemq.artemis.core.persistence.config.AbstractPersistedAddressSetting;
@@ -345,7 +348,7 @@ public class TransactionImplTest extends ServerTestBase {
       }
 
       @Override
-      public ArtemisCloseable closeableReadLock() {
+      public ArtemisCloseable closeableReadLock(boolean tryLock) {
          return () -> { };
       }
 
@@ -420,7 +423,7 @@ public class TransactionImplTest extends ServerTestBase {
       }
 
       @Override
-      public void pageWrite(SimpleString address, PagedMessage message, long pageNumber) {
+      public void pageWrite(SimpleString address, PagedMessage message, long pageNumber, boolean lineup, boolean originallyReplicated) {
 
       }
 
@@ -429,6 +432,11 @@ public class TransactionImplTest extends ServerTestBase {
          run.done();
       }
 
+      @Override
+      public void afterCompleteOperations(IOCallback run, OperationConsistencyLevel consistencyLevel) {
+         run.done();
+      }
+
       @Override
       public void afterStoreOperations(IOCallback run) {
          run.done();
@@ -459,20 +467,13 @@ public class TransactionImplTest extends ServerTestBase {
 
       }
 
-      @Override
-      public void confirmPendingLargeMessageTX(Transaction transaction,
-                                               long messageID,
-                                               long recordID) throws Exception {
-
-      }
-
       @Override
       public void injectMonitor(FileStoreMonitor monitor) throws Exception {
 
       }
 
       @Override
-      public void confirmPendingLargeMessage(long recordID) throws Exception {
+      public void deletePendingLargeMessage(long recordID) throws Exception {
 
       }
 
@@ -940,7 +941,31 @@ public class TransactionImplTest extends ServerTestBase {
 
       @Override
       public void deleteAddressStatus(long recordID) throws Exception {
+      }
 
+      @Override
+      public void storeMapRecord(long id,
+                                 byte recordType,
+                                 Persister persister,
+                                 Object record,
+                                 boolean sync,
+                                 IOCompletion completionCallback) throws Exception {
+      }
+
+      @Override
+      public void storeMapRecord(long id,
+                                 byte recordType,
+                                 Persister persister,
+                                 Object record,
+                                 boolean sync) throws Exception {
+      }
+
+      @Override
+      public void deleteMapRecord(long id, boolean sync) throws Exception {
+      }
+
+      @Override
+      public void deleteMapRecordTx(long txid, long id) throws Exception {
       }
    }
 
