@@ -23,16 +23,8 @@ import javax.jms.MessageNotWriteableException;
 
 import junit.framework.TestCase;
 
-/**
- *
- */
 public class ActiveMQStreamMessageTest extends TestCase {
 
-   /**
-    * Constructor for ActiveMQStreamMessageTest.
-    *
-    * @param name
-    */
    public ActiveMQStreamMessageTest(String name) {
       super(name);
    }
@@ -41,17 +33,11 @@ public class ActiveMQStreamMessageTest extends TestCase {
       junit.textui.TestRunner.run(ActiveMQStreamMessageTest.class);
    }
 
-   /*
-    * @see TestCase#setUp()
-    */
    @Override
    protected void setUp() throws Exception {
       super.setUp();
    }
 
-   /*
-    * @see TestCase#tearDown()
-    */
    @Override
    protected void tearDown() throws Exception {
       super.tearDown();
@@ -606,7 +592,7 @@ public class ActiveMQStreamMessageTest extends TestCase {
       ActiveMQStreamMessage msg = new ActiveMQStreamMessage();
       try {
          // Test with a 1Meg String
-         StringBuffer bigSB = new StringBuffer(1024 * 1024);
+         StringBuilder bigSB = new StringBuilder(1024 * 1024);
          for (int i = 0; i < 1024 * 1024; i++) {
             bigSB.append('a' + i % 26);
          }
