@@ -30,6 +30,7 @@ import javax.jms.Session;
 import javax.jms.TextMessage;
 import java.lang.invoke.MethodHandles;
 import java.util.HashMap;
+import java.util.Map;
 import java.util.UUID;
 import java.util.concurrent.TimeUnit;
 
@@ -44,7 +45,7 @@ import org.apache.activemq.artemis.core.server.transformer.Transformer;
 import org.apache.activemq.artemis.core.settings.impl.AddressFullMessagePolicy;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
 import org.apache.activemq.artemis.tests.util.CFUtil;
-import org.apache.activemq.artemis.tests.util.RandomUtil;
+import org.apache.activemq.artemis.utils.RandomUtil;
 import org.apache.activemq.artemis.tests.util.Wait;
 import org.apache.activemq.artemis.utils.collections.LinkedListIterator;
 import org.apache.activemq.transport.amqp.client.AmqpClient;
@@ -53,6 +54,7 @@ import org.apache.activemq.transport.amqp.client.AmqpMessage;
 import org.apache.activemq.transport.amqp.client.AmqpReceiver;
 import org.apache.activemq.transport.amqp.client.AmqpSender;
 import org.apache.activemq.transport.amqp.client.AmqpSession;
+import org.apache.qpid.proton.amqp.UnsignedInteger;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
 import org.slf4j.Logger;
@@ -199,8 +201,10 @@ public class AmqpExpiredMessageTest extends AmqpClientTestSupport {
       Wait.assertEquals(0, dlqView::getMessageCount);
    }
 
-   /** This test is validating a broker feature where the message copy through the DLQ will receive an annotation.
-    *  It is also testing filter on that annotation. */
+   /**
+    * This test is validating a broker feature where the message copy through the DLQ will receive an annotation. It is
+    * also testing filter on that annotation.
+    */
    @Test
    @Timeout(60)
    public void testExpiryThroughTTLValidateAnnotation() throws Exception {
@@ -272,8 +276,10 @@ public class AmqpExpiredMessageTest extends AmqpClientTestSupport {
       connection.close();
    }
 
-   /** This test is validating a broker feature where the message copy through the DLQ will receive an annotation.
-    *  It is also testing filter on that annotation. */
+   /**
+    * This test is validating a broker feature where the message copy through the DLQ will receive an annotation. It is
+    * also testing filter on that annotation.
+    */
    @Test
    @Timeout(60)
    public void testExpiryQpidJMS() throws Exception {
@@ -477,6 +483,55 @@ public class AmqpExpiredMessageTest extends AmqpClientTestSupport {
       connection.close();
    }
 
+   @Test
+   @Timeout(60)
+   public void testSendMessageThatIsNotExpiredUsingTimeToLiveOfMaxUInt() throws Exception {
+      doTestSendMessageThatIsNotExpiredUsingTimeToLive(UnsignedInteger.MAX_VALUE);
+   }
+
+   @Test
+   @Timeout(60)
+   public void testSendMessageThatIsNotExpiredUsingTimeToLiveOfMaxIntValue() throws Exception {
+      doTestSendMessageThatIsNotExpiredUsingTimeToLive(UnsignedInteger.valueOf(Integer.MAX_VALUE));
+   }
+
+   @Test
+   @Timeout(60)
+   public void testSendMessageThatIsNotExpiredUsingTimeToLiveOfMinusOne() throws Exception {
+      doTestSendMessageThatIsNotExpiredUsingTimeToLive(UnsignedInteger.valueOf(-1));
+   }
+
+   private void doTestSendMessageThatIsNotExpiredUsingTimeToLive(UnsignedInteger ttl) throws Exception {
+      AmqpClient client = createAmqpClient();
+      AmqpConnection connection = addConnection(client.connect());
+      AmqpSession session = connection.createSession();
+
+      AmqpSender sender = session.createSender(getQueueName());
+
+      // Get the Queue View early to avoid racing the delivery.
+      final Queue queueView = getProxyToQueue(getQueueName());
+      assertNotNull(queueView);
+
+      AmqpMessage message = new AmqpMessage();
+      message.setTimeToLive(ttl.longValue());
+      message.setText("Test-Message");
+      sender.send(message);
+      sender.close();
+
+      Wait.assertEquals(1, queueView::getMessageCount);
+
+      // Now try and get the message
+      AmqpReceiver receiver = session.createReceiver(getQueueName());
+      receiver.flow(1);
+      AmqpMessage received = receiver.receive(5, TimeUnit.SECONDS);
+      assertNotNull(received, "Should have read message but it seems to have timed out.");
+      assertEquals(ttl.longValue(), received.getTimeToLive());
+
+      Wait.assertEquals(0, queueView::getMessagesExpired);
+
+      connection.close();
+   }
+
    @Test
    @Timeout(60)
    public void testSendMessageThenAllowToExpiredUsingTimeToLive() throws Exception {
@@ -570,10 +625,10 @@ public class AmqpExpiredMessageTest extends AmqpClientTestSupport {
    @Test
    @Timeout(60)
    public void testExpirationAfterDivert() throws Throwable {
-      final String FORWARDING_ADDRESS = RandomUtil.randomString();
+      final String FORWARDING_ADDRESS = RandomUtil.randomUUIDString();
       server.createQueue(QueueConfiguration.of(FORWARDING_ADDRESS).setRoutingType(RoutingType.ANYCAST));
       server.deployDivert(new DivertConfiguration()
-                             .setName(RandomUtil.randomString())
+                             .setName(RandomUtil.randomUUIDString())
                              .setAddress(getQueueName())
                              .setForwardingAddress(FORWARDING_ADDRESS)
                              .setTransformerConfiguration(new TransformerConfiguration(MyTransformer.class.getName()))
@@ -799,7 +854,7 @@ public class AmqpExpiredMessageTest extends AmqpClientTestSupport {
 
       Wait.assertEquals(2, queueView::getMessageCount);
       LinkedListIterator<MessageReference> linkedListIterator = queueView.iterator();
-      HashMap<String, Long> dataSet = new HashMap<>();
+      Map<String, Long> dataSet = new HashMap<>();
       int count = 0;
       while (linkedListIterator.hasNext()) {
          count++;
@@ -829,13 +884,10 @@ public class AmqpExpiredMessageTest extends AmqpClientTestSupport {
          MessageReference ref = linkedListIterator.next();
          String idUsed = ref.getMessage().getStringProperty("id");
          long originalExpiration = dataSet.get(idUsed);
-         System.out.println("original Expiration = " + originalExpiration + " while this expiration = " + ref.getMessage().getExpiration());
+         logger.info("original Expiration = {} while this expiration = {}", originalExpiration, ref.getMessage().getExpiration());
          assertEquals(originalExpiration, ref.getMessage().getExpiration());
       }
       assertEquals(2, count);
       linkedListIterator.close();
-
-
    }
-
 }
