@@ -16,16 +16,19 @@
  */
 package org.apache.activemq.artemis.tests.integration.amqp.connect;
 
+import static org.apache.activemq.artemis.protocol.amqp.connect.bridge.AMQPBridgeConstants.PREFER_SHARED_DURABLE_SUBSCRIPTIONS;
 import static org.apache.activemq.artemis.protocol.amqp.connect.bridge.AMQPBridgeConstants.PULL_RECEIVER_BATCH_SIZE;
 import static org.apache.activemq.artemis.protocol.amqp.connect.bridge.AMQPBridgeConstants.RECEIVER_CREDITS;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.lang.invoke.MethodHandles;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.Map;
 
 import javax.jms.BytesMessage;
@@ -51,6 +54,7 @@ import org.apache.activemq.artemis.core.config.amqpBrokerConnectivity.AMQPBroker
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
 import org.apache.activemq.artemis.core.server.ComponentConfigurationRoutingType;
 import org.apache.activemq.artemis.core.server.impl.AddressInfo;
+import org.apache.activemq.artemis.protocol.amqp.broker.ProtonProtocolManagerFactory;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport;
 import org.apache.activemq.artemis.tests.integration.amqp.AmqpClientTestSupport;
 import org.apache.activemq.artemis.tests.util.CFUtil;
@@ -909,4 +913,474 @@ class AMQPBridgeServerToServerTest extends AmqpClientTestSupport {
          assertEquals(DeliveryMode.PERSISTENT, received.getJMSDeliveryMode());
       }
    }
+
+   @Test
+   @Timeout(20)
+   public void testBridgeToAddressPolicyDoesNotAccumulateMessagesByDefaultWhenConnectionDrops() throws Exception {
+      logger.info("Test started: {}", getTestName());
+
+      final AMQPBridgeAddressPolicyElement bridgeAddressPolicy = new AMQPBridgeAddressPolicyElement();
+      bridgeAddressPolicy.setName("test-policy");
+      bridgeAddressPolicy.addToIncludes(getTestName());
+
+      final AMQPBridgeBrokerConnectionElement element = new AMQPBridgeBrokerConnectionElement();
+      element.setName(getTestName());
+      element.addBridgeToAddressPolicy(bridgeAddressPolicy);
+
+      final AMQPBrokerConnectConfiguration amqpConnection =
+         new AMQPBrokerConnectConfiguration(getTestName(), "tcp://localhost:" + SERVER_PORT_REMOTE);
+      amqpConnection.setReconnectAttempts(10);// Limit reconnects
+      amqpConnection.setRetryInterval(50);
+      amqpConnection.addElement(element);
+
+      server.getConfiguration().addAMQPConnection(amqpConnection);
+      remoteServer.start();
+      server.start();
+
+      // We need to add the address before the bridge will start routing message to the remote.
+      server.addAddressInfo(new AddressInfo(SimpleString.of(getTestName()), RoutingType.MULTICAST));
+      // The bridge has been notified and has created a local consumer to bridge to the remote.
+      Wait.assertTrue(() -> server.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1, 5000, 50);
+
+      final ConnectionFactory factoryLocal = CFUtil.createConnectionFactory("AMQP", "tcp://localhost:" + SERVER_PORT);
+      final ConnectionFactory factoryRemote = CFUtil.createConnectionFactory("AMQP", "failover:(amqp://localhost:" + SERVER_PORT_REMOTE + ")");
+
+      try (Connection connectionL = factoryLocal.createConnection();
+           Connection connectionR = factoryRemote.createConnection()) {
+
+         final Session sessionL = connectionL.createSession(Session.AUTO_ACKNOWLEDGE);
+         final Session sessionR = connectionR.createSession(Session.AUTO_ACKNOWLEDGE);
+
+         final Topic topic = sessionL.createTopic(getTestName());
+
+         final MessageConsumer consumerR = sessionR.createConsumer(topic);
+
+         connectionL.start();
+         connectionR.start();
+
+         // Remote consumer is attached and ready for the bridged message
+         Wait.assertTrue(() -> remoteServer.addressQuery(SimpleString.of(getTestName())).isExists());
+         Wait.assertTrue(() -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1);
+
+         final MessageProducer producerL = sessionL.createProducer(topic);
+         final TextMessage message = sessionL.createTextMessage("Hello World");
+
+         message.setStringProperty("testProperty", "testValue-1");
+
+         producerL.send(message);
+
+         final Message received = consumerR.receive(5_000);
+         assertNotNull(received);
+         assertTrue(received instanceof TextMessage);
+         assertEquals("Hello World", ((TextMessage) received).getText());
+         assertTrue(received.propertyExists("testProperty"));
+         assertEquals("testValue-1", received.getStringProperty("testProperty"));
+
+         remoteServer.stop();
+
+         // The bridge has seen the connection drop and removed the local temporary address binding
+         Wait.assertTrue(() -> server.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 0, 5000, 50);
+
+         message.setStringProperty("testProperty", "testValue-2");
+
+         producerL.send(message);
+
+         remoteServer.start();
+
+         // Remote consumer is attached and ready for the next bridged message
+         Wait.assertTrue(() -> remoteServer.addressQuery(SimpleString.of(getTestName())).isExists());
+         Wait.assertTrue(() -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1);
+
+         // Connection has been restored to the remote and the bridge created a new binding to send to the remote
+         Wait.assertTrue(() -> server.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1, 5000, 50);
+
+         assertNull(consumerR.receiveNoWait());
+
+         message.setStringProperty("testProperty", "testValue-3");
+
+         producerL.send(message);
+
+         final Message receivedAfter = consumerR.receive(5_000);
+         assertNotNull(receivedAfter);
+         assertTrue(receivedAfter instanceof TextMessage);
+         assertEquals("Hello World", ((TextMessage) receivedAfter).getText());
+         assertTrue(receivedAfter.propertyExists("testProperty"));
+         assertEquals("testValue-3", receivedAfter.getStringProperty("testProperty"));
+      }
+   }
+
+   @Test
+   @Timeout(20)
+   public void testDurableBridgeToAddressPolicyAccumulatesMessagesWhenConnectionDrops() throws Exception {
+      logger.info("Test started: {}", getTestName());
+
+      final AMQPBridgeAddressPolicyElement bridgeAddressPolicy = new AMQPBridgeAddressPolicyElement();
+      bridgeAddressPolicy.setName("test-policy");
+      bridgeAddressPolicy.setUseDurableSubscriptions(true);
+      bridgeAddressPolicy.addToIncludes(getTestName());
+
+      final AMQPBridgeBrokerConnectionElement element = new AMQPBridgeBrokerConnectionElement();
+      element.setName(getTestName());
+      element.addBridgeToAddressPolicy(bridgeAddressPolicy);
+
+      final AMQPBrokerConnectConfiguration amqpConnection =
+         new AMQPBrokerConnectConfiguration(getTestName(), "tcp://localhost:" + SERVER_PORT_REMOTE);
+      amqpConnection.setReconnectAttempts(10);// Limit reconnects
+      amqpConnection.setRetryInterval(50);
+      amqpConnection.addElement(element);
+
+      server.getConfiguration().addAMQPConnection(amqpConnection);
+      remoteServer.start();
+      server.start();
+
+      // We need to add the address before the bridge will start routing message to the remote.
+      server.addAddressInfo(new AddressInfo(SimpleString.of(getTestName()), RoutingType.MULTICAST));
+      // The bridge has been notified and has created a local consumer to bridge to the remote.
+      Wait.assertTrue(() -> server.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1, 5000, 50);
+
+      // Remote server needs a durable queue to read messages from remote immediately after restart
+      remoteServer.createQueue(QueueConfiguration.of(getTestName()).setRoutingType(RoutingType.MULTICAST)
+                                                                   .setAddress(getTestName())
+                                                                   .setAutoCreated(false)
+                                                                   .setDurable(true));
+
+      final SimpleString subscriptionQueueName = server.bindingQuery(SimpleString.of(getTestName())).getQueueNames().get(0);
+
+      assertNotNull(subscriptionQueueName);
+      assertTrue(subscriptionQueueName.startsWith(SimpleString.of("amqp-bridge-")));
+
+      final org.apache.activemq.artemis.core.server.Queue subscriptionQueue = server.locateQueue(subscriptionQueueName);
+
+      assertNotNull(subscriptionQueue);
+
+      final ConnectionFactory factoryLocal = CFUtil.createConnectionFactory("AMQP", "tcp://localhost:" + SERVER_PORT);
+
+      try (Connection connectionL = factoryLocal.createConnection()) {
+         final Session sessionL = connectionL.createSession(Session.AUTO_ACKNOWLEDGE);
+         final Topic topic = sessionL.createTopic(getTestName());
+
+         connectionL.start();
+
+         // Remote consumer is attached and ready for the bridged message
+         Wait.assertTrue(() -> remoteServer.addressQuery(SimpleString.of(getTestName())).isExists());
+         Wait.assertTrue(() -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1);
+
+         final MessageProducer producerL = sessionL.createProducer(topic);
+         final TextMessage message = sessionL.createTextMessage("Hello World");
+
+         message.setStringProperty("testProperty", "testValue-1");
+         producerL.send(message);
+
+         Wait.assertEquals(0, () -> subscriptionQueue.getMessageCount());
+         Wait.assertEquals(1, () -> subscriptionQueue.getMessagesAcknowledged());
+
+         remoteServer.stop();
+
+         message.setStringProperty("testProperty", "testValue-2");
+         producerL.send(message);
+
+         // The bridge has seen the connection drop but the durable local binding remains
+         Wait.assertTrue(() -> server.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1);
+
+         remoteServer.start();
+
+         // Remote consumer is attached and ready for the next bridged message
+         Wait.assertTrue(() -> remoteServer.addressQuery(SimpleString.of(getTestName())).isExists());
+         Wait.assertTrue(() -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1);
+
+         message.setStringProperty("testProperty", "testValue-3");
+         producerL.send(message);
+
+         final SimpleString remoteBindingName = remoteServer.bindingQuery(SimpleString.of(getTestName())).getQueueNames().get(0);
+         assertNotNull(remoteBindingName);
+
+         final org.apache.activemq.artemis.core.server.Queue remoteQueueBinding = remoteServer.locateQueue(remoteBindingName);
+         assertNotNull(remoteQueueBinding);
+
+         Wait.assertEquals(3L, () -> remoteQueueBinding.getMessageCount(), 5_000, 100);
+      }
+   }
+
+   @Test
+   @Timeout(20)
+   public void testDurableReceiveFromAddressDropsOldMessagesOnSubscriptionUpdates() throws Exception {
+      logger.info("Test started: {}", getTestName());
+
+      final String filterStringA = "color='red'";
+      final String filterStringB = "color='green'";
+
+      final AMQPBridgeAddressPolicyElement bridgeAddressPolicy_1 = new AMQPBridgeAddressPolicyElement();
+      bridgeAddressPolicy_1.setName("test-policy");
+      bridgeAddressPolicy_1.setUseDurableSubscriptions(true);
+      bridgeAddressPolicy_1.setFilter(filterStringA);
+      bridgeAddressPolicy_1.addToIncludes(getTestName());
+
+      final AMQPBridgeAddressPolicyElement bridgeAddressPolicy_2 = new AMQPBridgeAddressPolicyElement();
+      bridgeAddressPolicy_2.setName("test-policy");
+      bridgeAddressPolicy_2.setUseDurableSubscriptions(true);
+      bridgeAddressPolicy_2.setFilter(filterStringB);
+      bridgeAddressPolicy_2.addToIncludes(getTestName());
+
+      final AMQPBridgeBrokerConnectionElement element = new AMQPBridgeBrokerConnectionElement();
+      element.setName(getTestName());
+      element.addBridgeFromAddressPolicy(bridgeAddressPolicy_1);
+
+      final AMQPBrokerConnectConfiguration amqpConnection =
+         new AMQPBrokerConnectConfiguration(getTestName(), "tcp://localhost:" + SERVER_PORT_REMOTE);
+      amqpConnection.setReconnectAttempts(10);// Limit reconnects
+      amqpConnection.setRetryInterval(50);
+      amqpConnection.addElement(element);
+
+      server.getConfiguration().addAMQPConnection(amqpConnection);
+      remoteServer.start();
+      server.start();
+
+      // Create an address with a binding to simulate demand from a consumer
+      server.createQueue(QueueConfiguration.of(getTestName()).setRoutingType(RoutingType.MULTICAST)
+                                                             .setAddress(getTestName())
+                                                             .setAutoCreated(false));
+      // Wait for the bridge to form to the remote and capture the durable subscription name
+      Wait.assertTrue(() -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1, 5_000, 50);
+
+      final String subscriptionQueueName = remoteServer.bindingQuery(SimpleString.of(getTestName())).getQueueNames().get(0).toString();
+
+      assertNotNull(subscriptionQueueName);
+      assertTrue(subscriptionQueueName.contains("amqp-bridge-"));
+      assertTrue(subscriptionQueueName.contains(getTestName()));
+
+      final org.apache.activemq.artemis.core.server.Queue subscriptionQueue = remoteServer.locateQueue(subscriptionQueueName);
+
+      assertNotNull(subscriptionQueue);
+      Wait.assertEquals(1L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+      assertTrue(subscriptionQueue.isDurable());
+      assertEquals(filterStringA, subscriptionQueue.getFilter().getFilterString().toString());
+
+      final ProtonProtocolManagerFactory protocolFactory = (ProtonProtocolManagerFactory)
+         server.getRemotingService().getProtocolFactoryMap().get("AMQP");
+      assertNotNull(protocolFactory);
+
+      final AMQPBridgeBrokerConnectionElement updatedElement = new AMQPBridgeBrokerConnectionElement();
+      updatedElement.setName(getTestName());
+      updatedElement.addBridgeFromAddressPolicy(bridgeAddressPolicy_2);
+
+      final AMQPBrokerConnectConfiguration updatedAmqpConnection =
+         new AMQPBrokerConnectConfiguration(getTestName(), "tcp://localhost:" + SERVER_PORT_REMOTE);
+      updatedAmqpConnection.setReconnectAttempts(10);// Limit reconnects
+      updatedAmqpConnection.setRetryInterval(50);
+      updatedAmqpConnection.addElement(updatedElement);
+
+      server.getConfiguration().getAMQPConnection().clear();
+      server.getConfiguration().addAMQPConnection(updatedAmqpConnection);
+
+      protocolFactory.updateProtocolServices(server, Collections.emptyList());
+
+      Wait.assertTrue(() -> {
+         return remoteServer.locateQueue(subscriptionQueueName).getFilter().getFilterString().toString().equals(filterStringB);
+      });
+
+      // There should be only one binding as the old one should have been replaced.
+      Wait.assertTrue(() -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size() == 1, 5_000, 50);
+   }
+
+   @Test
+   @Timeout(20)
+   public void testDurableAddressSubscriptionRecoveredOnRestart() throws Exception {
+      logger.info("Test started: {}", getTestName());
+
+      final String filterString = "color='red'";
+
+      final AMQPBridgeAddressPolicyElement bridgeAddressPolicy = new AMQPBridgeAddressPolicyElement();
+      bridgeAddressPolicy.setName("test-policy");
+      bridgeAddressPolicy.setUseDurableSubscriptions(true);
+      bridgeAddressPolicy.setFilter(filterString);
+      bridgeAddressPolicy.addToIncludes(getTestName());
+
+      final AMQPBridgeBrokerConnectionElement element = new AMQPBridgeBrokerConnectionElement();
+      element.setName(getTestName());
+      element.addBridgeFromAddressPolicy(bridgeAddressPolicy);
+
+      final AMQPBrokerConnectConfiguration amqpConnection =
+         new AMQPBrokerConnectConfiguration(getTestName(), "tcp://localhost:" + SERVER_PORT_REMOTE);
+      amqpConnection.setReconnectAttempts(10); // Limit reconnects
+      amqpConnection.setRetryInterval(50);
+      amqpConnection.addElement(element);
+
+      server.getConfiguration().addAMQPConnection(amqpConnection);
+      remoteServer.start();
+      server.start();
+
+      // Create an address with a binding to simulate demand from a consumer
+      server.createQueue(QueueConfiguration.of(getTestName()).setRoutingType(RoutingType.MULTICAST)
+                                                             .setAddress(getTestName())
+                                                             .setAutoCreated(false));
+      // Wait for the bridge to form to the remote and capture the durable subscription name
+      Wait.assertEquals(1L, () -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size(), 5_000, 50);
+
+      final String subscriptionQueueName = remoteServer.bindingQuery(SimpleString.of(getTestName())).getQueueNames().get(0).toString();
+
+      assertNotNull(subscriptionQueueName);
+      assertTrue(subscriptionQueueName.contains("amqp-bridge-"));
+      assertTrue(subscriptionQueueName.contains(getTestName()));
+
+      final org.apache.activemq.artemis.core.server.Queue subscriptionQueue = remoteServer.locateQueue(subscriptionQueueName);
+
+      assertNotNull(subscriptionQueue);
+      Wait.assertEquals(1L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+      assertTrue(subscriptionQueue.isDurable());
+      assertEquals(filterString, subscriptionQueue.getFilter().getFilterString().toString());
+
+      server.stop();
+
+      Wait.assertEquals(1L, () -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size(), 5_000, 50);
+      Wait.assertEquals(0L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+
+      final ConnectionFactory factoryRemote = CFUtil.createConnectionFactory("AMQP", "tcp://localhost:" + SERVER_PORT_REMOTE);
+
+      try (Connection connection = factoryRemote.createConnection()) {
+         final Session session = connection.createSession(Session.AUTO_ACKNOWLEDGE);
+         final Topic topic = session.createTopic(getTestName());
+         final MessageProducer producerL = session.createProducer(topic);
+         final TextMessage message = session.createTextMessage("Hello World");
+
+         message.setStringProperty("color", "green");
+         producerL.send(message);
+         message.setStringProperty("color", "red");
+         producerL.send(message);
+
+         Wait.assertEquals(1L, () -> subscriptionQueue.getMessageCount(), 5_000, 100);
+      }
+
+      server.start();
+
+      // Server should re-attach and recover the subscription and take the message
+      Wait.assertEquals(1L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+      Wait.assertEquals(0L, () -> subscriptionQueue.getMessageCount(), 5_000, 100);
+
+      final ConnectionFactory factoryLocal = CFUtil.createConnectionFactory("AMQP", "tcp://localhost:" + SERVER_PORT);
+
+      try (Connection connection = factoryLocal.createConnection()) {
+         final Session session = connection.createSession(Session.AUTO_ACKNOWLEDGE);
+         final Topic topic = session.createTopic(getTestName() + "::" + getTestName()); // Access our pre-created queue via FQQN
+         final MessageConsumer consumer = session.createConsumer(topic);
+
+         connection.start();
+
+         final Message receivedAfter = consumer.receive(5_000);
+
+         assertNotNull(receivedAfter);
+         assertTrue(receivedAfter instanceof TextMessage);
+         assertEquals("Hello World", ((TextMessage) receivedAfter).getText());
+         assertTrue(receivedAfter.propertyExists("color"));
+         assertEquals("red", receivedAfter.getStringProperty("color"));
+      }
+   }
+
+   @Test
+   @Timeout(20)
+   public void testSharedDurableAddressSubscriptionRecoveredOnRestart() throws Exception {
+      logger.info("Test started: {}", getTestName());
+
+      final String filterString = "color='red'";
+
+      final AMQPBridgeAddressPolicyElement bridgeAddressPolicy = new AMQPBridgeAddressPolicyElement();
+      bridgeAddressPolicy.setName("test-policy");
+      bridgeAddressPolicy.setUseDurableSubscriptions(true);
+      bridgeAddressPolicy.setFilter(filterString);
+      bridgeAddressPolicy.addToIncludes(getTestName());
+
+      final AMQPBridgeBrokerConnectionElement element = new AMQPBridgeBrokerConnectionElement();
+      element.setName(getTestName());
+      element.addBridgeFromAddressPolicy(bridgeAddressPolicy);
+      element.addProperty(PREFER_SHARED_DURABLE_SUBSCRIPTIONS, "true");
+
+      final AMQPBrokerConnectConfiguration amqpConnection =
+         new AMQPBrokerConnectConfiguration(getTestName(), "tcp://localhost:" + SERVER_PORT_REMOTE);
+      amqpConnection.setReconnectAttempts(10); // Limit reconnects
+      amqpConnection.setRetryInterval(50);
+      amqpConnection.addElement(element);
+
+      server.getConfiguration().addAMQPConnection(amqpConnection);
+      remoteServer.start();
+      server.start();
+
+      // Create an address with a binding to simulate demand from a consumer
+      server.createQueue(QueueConfiguration.of(getTestName()).setRoutingType(RoutingType.MULTICAST)
+                                                             .setAddress(getTestName())
+                                                             .setAutoCreated(false));
+      // Wait for the bridge to form to the remote and capture the durable subscription name
+      Wait.assertEquals(1L, () -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size(), 500_000, 50);
+
+      // The actual subscription queue for the "shared" bridge receivers should be a stable queue
+      final String subscriptionQueueName = remoteServer.bindingQuery(SimpleString.of(getTestName())).getQueueNames().get(0).toString();
+
+      assertNotNull(subscriptionQueueName);
+      assertTrue(subscriptionQueueName.contains("amqp-bridge-"));
+      assertTrue(subscriptionQueueName.contains(getTestName()));
+
+      final org.apache.activemq.artemis.core.server.Queue subscriptionQueue = remoteServer.locateQueue(subscriptionQueueName);
+
+      assertNotNull(subscriptionQueue);
+      Wait.assertEquals(1L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+      assertTrue(subscriptionQueue.isDurable());
+      assertEquals(filterString, subscriptionQueue.getFilter().getFilterString().toString());
+
+      server.stop();
+
+      Wait.assertEquals(1L, () -> remoteServer.bindingQuery(SimpleString.of(getTestName()), false).getQueueNames().size(), 5_000, 50);
+      Wait.assertEquals(0L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+
+      final ConnectionFactory factoryRemote = CFUtil.createConnectionFactory("AMQP", "tcp://localhost:" + SERVER_PORT_REMOTE);
+
+      try (Connection connection = factoryRemote.createConnection()) {
+         final Session session = connection.createSession(Session.AUTO_ACKNOWLEDGE);
+         final Topic topic = session.createTopic(getTestName());
+         final MessageProducer producerL = session.createProducer(topic);
+         final TextMessage message = session.createTextMessage("Hello World");
+
+         message.setStringProperty("color", "green");
+         producerL.send(message);
+         message.setStringProperty("color", "red");
+         producerL.send(message);
+
+         Wait.assertEquals(1L, () -> subscriptionQueue.getMessageCount(), 5_000, 100);
+      }
+
+      server.start();
+
+      // Server should re-attach and recover the subscription and take the message
+      Wait.assertEquals(1L, () -> subscriptionQueue.getConsumerCount(), 5_000, 100);
+      Wait.assertEquals(0L, () -> subscriptionQueue.getMessageCount(), 5_000, 100);
+
+      final ConnectionFactory factoryLocal = CFUtil.createConnectionFactory("AMQP", "tcp://localhost:" + SERVER_PORT);
+
+      try (Connection connection = factoryLocal.createConnection()) {
+         final Session session = connection.createSession(Session.AUTO_ACKNOWLEDGE);
+         final Topic topic = session.createTopic(getTestName() + "::" + getTestName()); // Access our pre-created queue via FQQN
+         final MessageConsumer consumer = session.createConsumer(topic);
+
+         connection.start();
+
+         final Message receivedAfter = consumer.receive(5_000);
+
+         assertNotNull(receivedAfter);
+         assertTrue(receivedAfter instanceof TextMessage);
+         assertEquals("Hello World", ((TextMessage) receivedAfter).getText());
+         assertTrue(receivedAfter.propertyExists("color"));
+         assertEquals("red", receivedAfter.getStringProperty("color"));
+
+         final TextMessage message = session.createTextMessage("Hello Again");
+         final MessageProducer producerL = session.createProducer(topic);
+
+         message.setStringProperty("color", "red");
+         producerL.send(message);
+
+         final Message receiveAnother = consumer.receive(5_000);
+
+         assertNotNull(receiveAnother);
+         assertTrue(receiveAnother instanceof TextMessage);
+         assertEquals("Hello Again", ((TextMessage) receiveAnother).getText());
+         assertTrue(receiveAnother.propertyExists("color"));
+         assertEquals("red", receiveAnother.getStringProperty("color"));
+      }
+   }
 }
