@@ -36,6 +36,7 @@ import org.apache.activemq.artemis.api.core.BroadcastEndpointFactory;
 import org.apache.activemq.artemis.api.core.JGroupsFileBroadcastEndpointFactory;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.TransportConfiguration;
+import org.apache.activemq.artemis.api.core.client.ActiveMQClient;
 import org.apache.activemq.artemis.api.core.jgroups.JChannelManager;
 import org.apache.activemq.artemis.api.core.management.CoreNotificationType;
 import org.apache.activemq.artemis.core.cluster.DiscoveryEntry;
@@ -56,22 +57,19 @@ import org.slf4j.LoggerFactory;
 
 /**
  * This will test Discovery test on JGroups and UDP.
- * <br>
- * In some configurations IPV6 may be a challenge. To make sure this test works, you may add this
- * property to your JVM settings: {@literal -Djgroups.bind_addr=::1}
- * <br>
+ * <p>
+ * In some configurations IPV6 may be a challenge. To make sure this test works, you may add this property to your JVM
+ * settings: {@literal -Djgroups.bind_addr=::1}
+ * <p>
  * Or ultimately you may also turn off IPV6: {@literal -Djava.net.preferIPv4Stack=true}
- * <br>
- * Note when you are not sure about your IP settings of your test machine, you should make sure
- * that the jgroups.bind_addr and java.net.preferXXStack by defining them explicitly, for example
- * if you would like to use IPV6, set BOTH properties to your JVM like the following:
- * -Djgroups.bind_addr=::1 -Djava.net.preferIPv6Addresses=true
- * <br>
- * or if you prefer IPV4:
- * -Djgroups.bind_addr=localhost -Djava.net.preferIPv4Stack=true
- * <br>
- * Also: Make sure you add integration-tests/src/tests/resources to your project path on the
- * tests/integration-tests
+ * <p>
+ * Note when you are not sure about your IP settings of your test machine, you should make sure that the
+ * jgroups.bind_addr and java.net.preferXXStack by defining them explicitly, for example if you would like to use IPV6,
+ * set BOTH properties to your JVM like the following: -Djgroups.bind_addr=::1 -Djava.net.preferIPv6Addresses=true
+ * <p>
+ * or if you prefer IPV4: -Djgroups.bind_addr=localhost -Djava.net.preferIPv4Stack=true
+ * <p>
+ * Also: Make sure you add integration-tests/src/tests/resources to your project path on the tests/integration-tests
  */
 public class DiscoveryTest extends DiscoveryBaseTest {
 
@@ -91,7 +89,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
    @AfterEach
    public void tearDown() throws Exception {
       JChannelManager.getInstance().clear().setLoopbackMessages(false);
-      /** This file path is defined at {@link #TEST_JGROUPS_CONF_FILE} */
+      // This file path is defined by TEST_JGROUPS_CONF_FILE
       deleteDirectory(new File("./target/tmp/amqtest.ping.dir"));
       for (ActiveMQComponent component : new ActiveMQComponent[]{bg, bg1, bg2, bg3, dg, dg1, dg2, dg3}) {
          stopComponent(component);
@@ -105,9 +103,9 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      final String nodeID = RandomUtil.randomString();
+      final String nodeID = RandomUtil.randomUUIDString();
 
-      bg = newBroadcast(nodeID, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(nodeID, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -115,7 +113,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
       dg.start();
 
@@ -127,7 +125,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
    @Test
    public void testSimpleBroadcastJGropus() throws Exception {
-      final String nodeID = RandomUtil.randomString();
+      final String nodeID = RandomUtil.randomUUIDString();
 
       bg = new BroadcastGroupImpl(new FakeNodeManager(nodeID), "broadcast", 100, null, new JGroupsFileBroadcastEndpointFactory().setChannelName("tst").setFile(TEST_JGROUPS_CONF_FILE));
 
@@ -137,7 +135,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg = new DiscoveryGroup(nodeID + "1", "broadcast", 5000L, new JGroupsFileBroadcastEndpointFactory().setChannelName("tst").setFile(TEST_JGROUPS_CONF_FILE), null);
+      dg = new DiscoveryGroup(nodeID + "1", "broadcast", 5000L, ActiveMQClient.DEFAULT_DISCOVERY_STOPPING_TIMEOUT, new JGroupsFileBroadcastEndpointFactory().setChannelName("tst").setFile(TEST_JGROUPS_CONF_FILE), null);
 
       dg.start();
 
@@ -156,10 +154,8 @@ public class DiscoveryTest extends DiscoveryBaseTest {
    }
 
    /**
-    * Create one broadcaster and 100 receivers. Make sure broadcasting works.
-    * Then stop 99 of the receivers, the last one could still be working.
-    *
-    * @throws Exception
+    * Create one broadcaster and 100 receivers. Make sure broadcasting works. Then stop 99 of the receivers, the last
+    * one could still be working.
     */
    @Test
    public void testJGropusChannelReferenceCounting() throws Exception {
@@ -215,11 +211,8 @@ public class DiscoveryTest extends DiscoveryBaseTest {
    }
 
    /**
-    * Create one broadcaster and 50 receivers. Make sure broadcasting works.
-    * Then stop all of the receivers, and create 50 new ones. Make sure the
-    * 50 new ones are receiving data from the broadcasting.
-    *
-    * @throws Exception
+    * Create one broadcaster and 50 receivers. Make sure broadcasting works. Then stop all of the receivers, and create
+    * 50 new ones. Make sure the 50 new ones are receiving data from the broadcasting.
     */
    @Test
    public void testJGropusChannelReferenceCounting1() throws Exception {
@@ -281,10 +274,8 @@ public class DiscoveryTest extends DiscoveryBaseTest {
    }
 
    /**
-    * Create one broadcaster and 50 receivers. Then stop half of the receivers.
-    * Then add the half back, plus some more. Make sure all receivers receive data.
-    *
-    * @throws Exception
+    * Create one broadcaster and 50 receivers. Then stop half of the receivers. Then add the half back, plus some more.
+    * Make sure all receivers receive data.
     */
    @Test
    public void testJGropusChannelReferenceCounting2() throws Exception {
@@ -403,7 +394,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      final String nodeID = RandomUtil.randomString();
+      final String nodeID = RandomUtil.randomUUIDString();
 
       // We need to choose a real NIC on the local machine - note this will silently pass if the machine
       // has no usable NIC!
@@ -414,7 +405,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       logger.debug("Local address is {}", localAddress);
 
-      bg = newBroadcast(nodeID, RandomUtil.randomString(), localAddress, -1, groupAddress, groupPort);
+      bg = newBroadcast(nodeID, RandomUtil.randomUUIDString(), localAddress, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -422,7 +413,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), localAddress, groupAddress, groupPort, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), localAddress, groupAddress, groupPort, timeout);
 
       dg.start();
 
@@ -439,9 +430,9 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      final String nodeID = RandomUtil.randomString();
+      final String nodeID = RandomUtil.randomUUIDString();
 
-      bg = newBroadcast(nodeID, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(nodeID, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -449,7 +440,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
       dg.start();
 
@@ -476,9 +467,9 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      String nodeID = RandomUtil.randomString();
+      String nodeID = RandomUtil.randomUUIDString();
 
-      bg = newBroadcast(nodeID, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(nodeID, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -486,7 +477,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg = newDiscoveryGroup(nodeID, RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg = newDiscoveryGroup(nodeID, RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
       dg.start();
 
@@ -505,7 +496,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      bg = newBroadcast(RandomUtil.randomString(), RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -515,7 +506,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       final int port2 = getUDPDiscoveryPort(1);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, port2, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, port2, timeout);
 
       dg.start();
 
@@ -528,7 +519,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      bg = newBroadcast(RandomUtil.randomString(), RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -539,7 +530,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final InetAddress groupAddress2 = InetAddress.getByName(address2);
       final int port2 = getUDPDiscoveryPort(1);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress2, port2, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress2, port2, timeout);
 
       dg.start();
 
@@ -568,11 +559,11 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       String node3 = UUIDGenerator.getInstance().generateStringUUID();
 
-      bg1 = newBroadcast(node1, RandomUtil.randomString(), null, -1, groupAddress1, groupPort1);
+      bg1 = newBroadcast(node1, RandomUtil.randomUUIDString(), null, -1, groupAddress1, groupPort1);
 
-      bg2 = newBroadcast(node2, RandomUtil.randomString(), null, -1, groupAddress2, groupPort2);
+      bg2 = newBroadcast(node2, RandomUtil.randomUUIDString(), null, -1, groupAddress2, groupPort2);
 
-      bg3 = newBroadcast(node3, RandomUtil.randomString(), null, -1, groupAddress3, groupPort3);
+      bg3 = newBroadcast(node3, RandomUtil.randomUUIDString(), null, -1, groupAddress3, groupPort3);
 
       bg2.start();
       bg1.start();
@@ -588,13 +579,13 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       bg2.addConnector(live2);
       bg3.addConnector(live3);
 
-      dg1 = newDiscoveryGroup("group-1::" + RandomUtil.randomString(), "group-1::" + RandomUtil.randomString(), null, groupAddress1, groupPort1, timeout);
+      dg1 = newDiscoveryGroup("group-1::" + RandomUtil.randomUUIDString(), "group-1::" + RandomUtil.randomUUIDString(), null, groupAddress1, groupPort1, timeout);
       dg1.start();
 
-      dg2 = newDiscoveryGroup("group-2::" + RandomUtil.randomString(), "group-2::" + RandomUtil.randomString(), null, groupAddress2, groupPort2, timeout);
+      dg2 = newDiscoveryGroup("group-2::" + RandomUtil.randomUUIDString(), "group-2::" + RandomUtil.randomUUIDString(), null, groupAddress2, groupPort2, timeout);
       dg2.start();
 
-      dg3 = newDiscoveryGroup("group-3::" + RandomUtil.randomString(), "group-3::" + RandomUtil.randomString(), null, groupAddress3, groupPort3, timeout);
+      dg3 = newDiscoveryGroup("group-3::" + RandomUtil.randomUUIDString(), "group-3::" + RandomUtil.randomUUIDString(), null, groupAddress3, groupPort3, timeout);
       dg3.start();
 
       bg1.broadcastConnectors();
@@ -669,9 +660,9 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      String nodeID = RandomUtil.randomString();
+      String nodeID = RandomUtil.randomUUIDString();
 
-      bg = newBroadcast(nodeID, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(nodeID, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -679,7 +670,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
       MyListener listener1 = new MyListener();
       MyListener listener2 = new MyListener();
@@ -715,17 +706,17 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      String node1 = RandomUtil.randomString();
-      String node2 = RandomUtil.randomString();
-      String node3 = RandomUtil.randomString();
+      String node1 = RandomUtil.randomUUIDString();
+      String node2 = RandomUtil.randomUUIDString();
+      String node3 = RandomUtil.randomUUIDString();
 
-      bg1 = newBroadcast(node1, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg1 = newBroadcast(node1, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
       bg1.start();
 
-      bg2 = newBroadcast(node2, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg2 = newBroadcast(node2, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
       bg2.start();
 
-      bg3 = newBroadcast(node3, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg3 = newBroadcast(node3, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
       bg3.start();
 
       TransportConfiguration live1 = generateTC();
@@ -737,7 +728,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       TransportConfiguration live3 = generateTC();
       bg3.addConnector(live3);
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
       MyListener listener1 = new MyListener();
       dg.registerListener(listener1);
@@ -862,9 +853,9 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      String nodeID = RandomUtil.randomString();
+      String nodeID = RandomUtil.randomUUIDString();
 
-      bg = newBroadcast(nodeID, RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(nodeID, RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.start();
 
@@ -872,11 +863,11 @@ public class DiscoveryTest extends DiscoveryBaseTest {
 
       bg.addConnector(live1);
 
-      dg1 = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg1 = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
-      dg2 = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg2 = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
-      dg3 = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout);
+      dg3 = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout);
 
       dg1.start();
       dg2.start();
@@ -916,7 +907,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final int groupPort = getUDPDiscoveryPort();
       final int timeout = 500;
 
-      dg = newDiscoveryGroup(RandomUtil.randomString(), RandomUtil.randomString(), null, groupAddress, groupPort, timeout, notifService);
+      dg = newDiscoveryGroup(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, groupAddress, groupPort, timeout, notifService);
 
       assertEquals(0, notifListener.count(CoreNotificationType.DISCOVERY_GROUP_STARTED));
 
@@ -944,7 +935,7 @@ public class DiscoveryTest extends DiscoveryBaseTest {
       final InetAddress groupAddress = InetAddress.getByName(address1);
       final int groupPort = getUDPDiscoveryPort();
 
-      bg = newBroadcast(RandomUtil.randomString(), RandomUtil.randomString(), null, -1, groupAddress, groupPort);
+      bg = newBroadcast(RandomUtil.randomUUIDString(), RandomUtil.randomUUIDString(), null, -1, groupAddress, groupPort);
 
       bg.setNotificationService(notifService);
 
