@@ -21,6 +21,7 @@ import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertSame;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.InputStream;
@@ -74,11 +75,11 @@ import org.junit.jupiter.api.Test;
 
 public class RedeployTest extends ActiveMQTestBase {
 
-   @Test
    /*
     * This tests that the broker doesnt fall over when it tries to delete any autocreated addresses/queues in a clustered environment
     * If the undeploy fails then bridges etc can stop working, we need to make sure if undeploy fails on anything the broker is still live
-    * */
+    */
+   @Test
    public void testRedeployAutoCreateAddress() throws Exception {
       Path brokerXML = getTestDirfile().toPath().resolve("broker.xml");
       URL url1 = RedeployTest.class.getClassLoader().getResource("reload-test-autocreateaddress.xml");
@@ -503,7 +504,7 @@ public class RedeployTest extends ActiveMQTestBase {
 
          Binding bindingAfterChange = embeddedActiveMQ.getActiveMQServer().getPostOffice().getBinding(SimpleString.of("myFilterQueue"));
 
-         assertTrue(binding == bindingAfterChange, "Instance should be the same (as should be non destructive)");
+         assertSame(binding, bindingAfterChange, "Instance should be the same (as should be non destructive)");
          assertEquals(binding.getID(), bindingAfterChange.getID());
 
          //Check that after the config change we can still consume a message that was sent before, ensuring config change was non-destructive of the queue.
@@ -589,9 +590,9 @@ public class RedeployTest extends ActiveMQTestBase {
             Queue queue = session.createQueue("a-from");
             MessageProducer producer = session.createProducer(queue);
             producer.send(session.createMessage());
-            Wait.assertEquals(1, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("a-new").getMessageCount());
-            Wait.assertEquals(1, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("a-to").getMessageCount());
-            Wait.assertEquals(2, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("a-from").getConsumerCount());
+            Wait.assertEquals(1, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("a-new").getMessageCount(), 2000);
+            Wait.assertEquals(1, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("a-to").getMessageCount(), 2000);
+            Wait.assertEquals(2, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("a-from").getConsumerCount(), 2000);
          }
 
          try (ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory();
@@ -609,7 +610,7 @@ public class RedeployTest extends ActiveMQTestBase {
             Queue queue = session.createQueue("c-from");
             MessageProducer producer = session.createProducer(queue);
             producer.send(session.createMessage());
-            Wait.assertEquals(1, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("c-to").getMessageCount());
+            Wait.assertEquals(1, () -> embeddedActiveMQ.getActiveMQServer().locateQueue("c-to").getMessageCount(), 2000);
          }
 
       } finally {
@@ -721,7 +722,8 @@ public class RedeployTest extends ActiveMQTestBase {
 
    /**
     * This one is here just to make sure it's possible to change queue parameters one by one without setting the others
-    * to <code>null</code>.
+    * to {@code null}.
+    *
     * @throws Exception An exception.
     */
    @Test
@@ -1191,7 +1193,7 @@ public class RedeployTest extends ActiveMQTestBase {
 
       ConnectionFactory connectionFactory = new ActiveMQConnectionFactory();
       try (JMSContext jmsContext = connectionFactory.createContext()) {
-         jmsContext.createSharedDurableConsumer(jmsContext.createTopic("config_test_consumer_created_queues"),"mySub").receive(100);
+         jmsContext.createSharedDurableConsumer(jmsContext.createTopic("config_test_consumer_created_queues"), "mySub").receive(100);
       }
 
       try {
@@ -1340,8 +1342,8 @@ public class RedeployTest extends ActiveMQTestBase {
    }
 
    /**
-    * Simulates Stop and Start that occurs when network health checker stops the server when network is detected unhealthy
-    * and re-starts the broker once detected that it is healthy again.
+    * Simulates Stop and Start that occurs when network health checker stops the server when network is detected
+    * unhealthy and re-starts the broker once detected that it is healthy again.
     *
     * @throws Exception for anything un-expected, test will fail.
     */
@@ -1365,8 +1367,8 @@ public class RedeployTest extends ActiveMQTestBase {
       try {
          latch.await(10, TimeUnit.SECONDS);
 
-         assertEquals(getSecurityRoles(embeddedActiveMQ, "security_address").size(), 1);
-         assertEquals(getSecurityRoles(embeddedActiveMQ, "security_address").iterator().next().getName(), "b");
+         assertEquals(1, getSecurityRoles(embeddedActiveMQ, "security_address").size());
+         assertEquals("b", getSecurityRoles(embeddedActiveMQ, "security_address").iterator().next().getName());
 
          assertEquals(getAddressSettings(embeddedActiveMQ, "address_settings_address").getDeadLetterAddress(), SimpleString.of("OriginalDLQ"));
          assertEquals(getAddressSettings(embeddedActiveMQ, "address_settings_address").getExpiryAddress(), SimpleString.of("OriginalExpiryQueue"));
@@ -1393,8 +1395,8 @@ public class RedeployTest extends ActiveMQTestBase {
          latch.await(10, TimeUnit.SECONDS);
 
          //Assert that the security settings change applied
-         assertEquals(getSecurityRoles(embeddedActiveMQ, "security_address").size(), 1);
-         assertEquals(getSecurityRoles(embeddedActiveMQ, "security_address").iterator().next().getName(), "c");
+         assertEquals(1, getSecurityRoles(embeddedActiveMQ, "security_address").size());
+         assertEquals("c", getSecurityRoles(embeddedActiveMQ, "security_address").iterator().next().getName());
 
          //Assert that the address settings change applied
          assertEquals(getAddressSettings(embeddedActiveMQ, "address_settings_address").getDeadLetterAddress(), SimpleString.of("NewDLQ"));
@@ -1425,8 +1427,8 @@ public class RedeployTest extends ActiveMQTestBase {
          embeddedActiveMQ.start();
 
          //Assert that the security settings changes persist a stop and start server (e.g. like what occurs if network health check stops the node), but JVM remains up.
-         assertEquals(getSecurityRoles(embeddedActiveMQ, "security_address").size(), 1);
-         assertEquals(getSecurityRoles(embeddedActiveMQ, "security_address").iterator().next().getName(), "c");
+         assertEquals(1, getSecurityRoles(embeddedActiveMQ, "security_address").size());
+         assertEquals("c", getSecurityRoles(embeddedActiveMQ, "security_address").iterator().next().getName());
 
          //Assert that the address settings changes persist a stop and start server (e.g. like what occurs if network health check stops the node), but JVM remains up.
          assertEquals(getAddressSettings(embeddedActiveMQ, "address_settings_address").getDeadLetterAddress(), SimpleString.of("NewDLQ"));
