@@ -44,6 +44,7 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertSame;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 public class PingTest extends ActiveMQTestBase {
@@ -83,7 +84,7 @@ public class PingTest extends ActiveMQTestBase {
       }
    }
 
-   /*
+   /**
     * Test that no failure listeners are triggered in a non failure case with pinging going on
     */
    @Test
@@ -127,7 +128,7 @@ public class PingTest extends ActiveMQTestBase {
 
       RemotingConnection serverConn2 = server.getRemotingService().getConnections().iterator().next();
 
-      assertTrue(serverConn == serverConn2);
+      assertSame(serverConn, serverConn2);
 
       session.close();
 
@@ -136,7 +137,7 @@ public class PingTest extends ActiveMQTestBase {
       locator.close();
    }
 
-   /*
+   /**
     * Test that no failure listeners are triggered in a non failure case with no pinging going on
     */
    @Test
@@ -179,7 +180,7 @@ public class PingTest extends ActiveMQTestBase {
 
       RemotingConnection serverConn2 = server.getRemotingService().getConnections().iterator().next();
 
-      assertTrue(serverConn == serverConn2);
+      assertSame(serverConn, serverConn2);
 
       session.close();
 
@@ -188,7 +189,7 @@ public class PingTest extends ActiveMQTestBase {
       locator.close();
    }
 
-   /*
+   /**
     * Test that pinging is disabled for in-vm connection when using the default settings
     */
    @Test
@@ -225,7 +226,7 @@ public class PingTest extends ActiveMQTestBase {
       locator.close();
    }
 
-   /*
+   /**
     * Test the server timing out a connection since it doesn't receive a ping in time
     */
    @Test
@@ -294,9 +295,9 @@ public class PingTest extends ActiveMQTestBase {
       locator.close();
    }
 
-   /*
-   * Test the client triggering failure due to no ping from server received in time
-   */
+   /**
+    * Test the client triggering failure due to no ping from server received in time
+    */
    @Test
    public void testClientFailureNoServerPing() throws Exception {
       // server must received at least one ping from the client to pass
