@@ -18,7 +18,9 @@ package org.apache.activemq.artemis.tests.integration.replication;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 
@@ -166,7 +168,7 @@ public final class ReplicationTest extends ActiveMQTestBase {
 
       Configuration primaryConfig = createDefaultInVMConfig();
 
-      Configuration backupConfig = createDefaultInVMConfig().setHAPolicyConfiguration(new SharedStoreBackupPolicyConfiguration()).setBindingsDirectory(getBindingsDir(0, true)).setJournalDirectory(getJournalDir(0, true)).setPagingDirectory(getPageDir(0, true)).setLargeMessagesDirectory(getLargeMessagesDir(0, true)).setIncomingInterceptorClassNames(incomingInterceptors.length > 0 ? Arrays.asList(incomingInterceptors) : new ArrayList<String>());
+      Configuration backupConfig = createDefaultInVMConfig().setHAPolicyConfiguration(new SharedStoreBackupPolicyConfiguration()).setBindingsDirectory(getBindingsDir(0, true)).setJournalDirectory(getJournalDir(0, true)).setPagingDirectory(getPageDir(0, true)).setLargeMessagesDirectory(getLargeMessagesDir(0, true)).setIncomingInterceptorClassNames(incomingInterceptors.length > 0 ? Arrays.asList(incomingInterceptors) : new ArrayList<>());
 
       if (!pluggableQuorum) {
          ReplicatedBackupUtils.configureReplicationPair(backupConfig, backupConnector, backupAcceptor, primaryConfig, primaryConnector, primaryAcceptor);
@@ -275,14 +277,14 @@ public final class ReplicationTest extends ActiveMQTestBase {
       replicatedJournal.appendAddRecordTransactional(23, 24, (byte) 1, new FakeData());
 
       PagedMessage pgmsg = new PagedMessageImpl(msg, new long[0]);
-      manager.pageWrite(pgmsg, 1);
-      manager.pageWrite(pgmsg, 2);
-      manager.pageWrite(pgmsg, 3);
-      manager.pageWrite(pgmsg, 4);
+      manager.pageWrite(dummy, pgmsg, 1, true);
+      manager.pageWrite(dummy, pgmsg, 2, true);
+      manager.pageWrite(dummy, pgmsg, 3, true);
+      manager.pageWrite(dummy, pgmsg, 4, true);
 
       blockOnReplication(storage, manager);
 
-      PagingManager pagingManager = createPageManager(backupServer.getStorageManager(), backupServer.getConfiguration(), backupServer.getExecutorFactory(), backupServer.getAddressSettingsRepository());
+      PagingManager pagingManager = createPageManager(backupServer.getStorageManager(), backupServer.getConfiguration(), backupServer.getExecutorFactory(), backupServer.getScheduledPool(), backupServer.getAddressSettingsRepository());
 
       PagingStore store = pagingManager.getPageStore(dummy);
       store.start();
@@ -362,6 +364,62 @@ public final class ReplicationTest extends ActiveMQTestBase {
       }
    }
 
+   @TestTemplate
+   public void testSendMessageWithLargeHeader() throws Exception {
+      setupServer(true, TestInterceptor.class.getName());
+
+      manager = liveServer.getReplicationManager();
+      waitForComponent(manager);
+
+      ClientSessionFactory sf = createSessionFactory(locator);
+
+      try (ClientSession producerSession = sf.createSession();
+           ClientSession consumerSession = sf.createSession()) {
+
+         producerSession.start();
+         producerSession.createQueue(QueueConfiguration.of(ADDRESS));
+         ClientProducer producer = producerSession.createProducer(ADDRESS);
+
+         consumerSession.start();
+         ClientConsumer consumer = consumerSession.createConsumer(ADDRESS);
+
+         ClientMessage messageBefore = producerSession.createMessage(true);
+         setBody(0, messageBefore);
+         messageBefore.putIntProperty("counter", 0);
+         producer.send(messageBefore);
+
+         ClientMessage messageReceivedBefore = consumer.receive(1000);
+         assertNotNull(messageReceivedBefore, "Message should exist!");
+         assertMessageBody(0, messageReceivedBefore);
+         assertEquals(0, messageReceivedBefore.getIntProperty("counter").intValue());
+         messageReceivedBefore.acknowledge();
+
+         ClientMessage messageWithLargeHeader = producerSession.createMessage(true);
+         setBody(1, messageWithLargeHeader);
+         messageWithLargeHeader.putIntProperty("counter", 1);
+         messageWithLargeHeader.putStringProperty("large-property", "z".repeat(512 * 1024));
+         try {
+            producer.send(messageWithLargeHeader);
+            fail();
+         } catch (Exception e) {
+            assertTrue(e.getMessage().contains("AMQ149005"));
+         }
+
+         ClientMessage messageAfter = producerSession.createMessage(true);
+         setBody(2, messageAfter);
+         messageAfter.putIntProperty("counter", 2);
+         producer.send(messageAfter);
+
+         ClientMessage messageReceivedAfter = consumer.receive(1000);
+         assertNotNull(messageReceivedAfter, "Message should exist!");
+         assertMessageBody(2, messageReceivedAfter);
+         assertEquals(2, messageReceivedAfter.getIntProperty("counter").intValue());
+         messageReceivedAfter.acknowledge();
+
+         assertNull(consumer.receiveImmediate());
+      }
+   }
+
    @TestTemplate
    public void testExceptionSettingActionBefore() throws Exception {
       OperationContext ctx = OperationContextImpl.getContext(factory);
@@ -445,19 +503,15 @@ public final class ReplicationTest extends ActiveMQTestBase {
       final long ttlOverride = 123456789;
       final long checkPeriodOverride = 987654321;
 
-      ExtraConfigurer configurer = new ExtraConfigurer() {
-
-         @Override
-         public void config(Configuration primaryConfig, Configuration backupConfig) {
-            List<ClusterConnectionConfiguration> ccList = backupConfig.getClusterConfigurations();
-            assertTrue(ccList.size() > 0);
-            ClusterConnectionConfiguration cc = ccList.get(0);
-            cc.setConnectionTTL(ttlOverride);
-            cc.setClientFailureCheckPeriod(checkPeriodOverride);
-         }
+      ExtraConfigurer configurer = (primaryConfig, backupConfig) -> {
+         List<ClusterConnectionConfiguration> ccList = backupConfig.getClusterConfigurations();
+         assertFalse(ccList.isEmpty());
+         ClusterConnectionConfiguration cc = ccList.get(0);
+         cc.setConnectionTTL(ttlOverride);
+         cc.setClientFailureCheckPeriod(checkPeriodOverride);
       };
       this.setupServer(true, true, configurer);
-      assertTrue(backupServer instanceof ActiveMQServerImpl);
+      assertInstanceOf(ActiveMQServerImpl.class, backupServer);
 
       ClusterController controller = backupServer.getClusterManager().getClusterController();
 
@@ -469,17 +523,10 @@ public final class ReplicationTest extends ActiveMQTestBase {
       assertEquals(checkPeriodOverride, replicationLocator.getClientFailureCheckPeriod());
    }
 
-   /**
-    * @return
-    * @throws Exception
-    */
    private JournalStorageManager getStorage() throws Exception {
       return new JournalStorageManager(createDefaultInVMConfig(), EmptyCriticalAnalyzer.getInstance(), factory, factory);
    }
 
-   /**
-    * @param manager1
-    */
    private void blockOnReplication(final StorageManager storage, final ReplicationManager manager1) throws Exception {
       final CountDownLatch latch = new CountDownLatch(1);
       storage.afterCompleteOperations(new IOCallback() {
@@ -637,8 +684,8 @@ public final class ReplicationTest extends ActiveMQTestBase {
    }
 
    /**
-    * We need to shutdown the executors before calling {@link super#tearDown()} (which will check
-    * for leaking threads). Due to that, we need to close/stop all components here.
+    * We need to shutdown the executors before calling {@link super#tearDown()} (which will check for leaking threads).
+    * Due to that, we need to close/stop all components here.
     */
    @Override
    @AfterEach
@@ -663,9 +710,10 @@ public final class ReplicationTest extends ActiveMQTestBase {
    protected PagingManager createPageManager(final StorageManager storageManager,
                                              final Configuration configuration,
                                              final ExecutorFactory executorFactory,
+                                             final ScheduledExecutorService scheduledExecutorService,
                                              final HierarchicalRepository<AddressSettings> addressSettingsRepository) throws Exception {
 
-      PagingManager paging = new PagingManagerImpl(new PagingStoreFactoryNIO(storageManager, configuration.getPagingLocation(), 1000, null, executorFactory, executorFactory, false, null), addressSettingsRepository, configuration.getManagementAddress());
+      PagingManager paging = new PagingManagerImpl(new PagingStoreFactoryNIO(storageManager, configuration.getPagingLocation(), 1000, scheduledExecutorService, executorFactory, false, null), addressSettingsRepository, configuration.getManagementAddress());
 
       paging.start();
       runAfter(paging::stop);
